@@ -93,6 +93,7 @@ from main import events
 from main import config
 
 from crypt import my_keys
+from crypt import signed
 
 from dht import dht_relations
 
@@ -129,8 +130,6 @@ _ActiveGroupMembersByIDURL = {}
 #------------------------------------------------------------------------------
 
 def register_group_member(A):
-    """
-    """
     global _ActiveGroupMembers
     global _ActiveGroupMembersByIDURL
     if _Debug:
@@ -146,8 +145,6 @@ def register_group_member(A):
 
 
 def unregister_group_member(A):
-    """
-    """
     global _ActiveGroupMembers
     global _ActiveGroupMembersByIDURL
     if _Debug:
@@ -166,15 +163,11 @@ def unregister_group_member(A):
 #------------------------------------------------------------------------------
 
 def list_active_group_members():
-    """
-    """
     global _ActiveGroupMembers
     return list(_ActiveGroupMembers.keys())
 
 
 def get_active_group_member(group_key_id):
-    """
-    """
     global _ActiveGroupMembers
     if group_key_id not in _ActiveGroupMembers:
         return None
@@ -182,8 +175,6 @@ def get_active_group_member(group_key_id):
 
 
 def find_active_group_members(group_creator_idurl):
-    """
-    """
     global _ActiveGroupMembersByIDURL
     result = []
     for automat_index in _ActiveGroupMembersByIDURL.values():
@@ -346,7 +337,7 @@ class GroupMember(automat.Automat):
         self.member_idurl = member_idurl or my_id.getIDURL()
         self.member_id = self.member_idurl.to_id()
         self.group_key_id = group_key_id
-        self.group_glob_id = global_id.ParseGlobalID(self.group_key_id)
+        self.group_glob_id = global_id.NormalizeGlobalID(self.group_key_id)
         self.group_queue_alias = self.group_glob_id['key_alias']
         self.group_creator_id = self.group_glob_id['customer']
         self.group_creator_idurl = self.group_glob_id['idurl']
@@ -364,7 +355,7 @@ class GroupMember(automat.Automat):
         self.recorded_messages = []
         self.group_brokers_updated = False
         super(GroupMember, self).__init__(
-            name="group_member_%s$%s" % (self.group_queue_alias[:10], self.group_creator_id),
+            name="group_member_%s$%s" % (self.group_queue_alias, self.group_creator_id, ),
             state="AT_STARTUP",
             debug_level=debug_level,
             log_events=log_events,
@@ -381,7 +372,7 @@ class GroupMember(automat.Automat):
         if _Debug:
             lg.args(_DebugLevel, old=self.group_key_id, new=new_group_key_id)
         self.group_key_id = new_group_key_id
-        self.group_glob_id = global_id.ParseGlobalID(self.group_key_id)
+        self.group_glob_id = global_id.NormalizeGlobalID(self.group_key_id)
         self.group_queue_alias = self.group_glob_id['key_alias']
         self.group_creator_id = self.group_glob_id['customer']
         self.group_creator_idurl = self.group_glob_id['idurl']
@@ -394,7 +385,7 @@ class GroupMember(automat.Automat):
             'member_id': self.member_id,
             'group_key_id': self.group_key_id,
             'alias': self.group_glob_id['key_alias'],
-            'label': my_keys.get_label(self.group_key_id),
+            'label': my_keys.get_label(self.group_key_id) or '',
             'creator': self.group_creator_id,
             'active_broker_id': self.active_broker_id,
             'active_queue_id': self.active_queue_id,
@@ -1349,7 +1340,7 @@ class GroupMember(automat.Automat):
                 evt, a, kw = err.value.args
                 if a and a[0]:
                     if isinstance(a[0], packet_out.PacketOut):
-                        resp_payload = strng.to_text(a[0].Payload)
+                        resp_payload = strng.to_text(a[0].outpacket.Payload)
                     else:
                         resp_payload = strng.to_text(a[0][0].Payload)
                     if resp_payload.startswith('identity:'):
@@ -1401,7 +1392,7 @@ class GroupMember(automat.Automat):
                 evt, a, kw = err.value.args
                 if a and a[0]:
                     if isinstance(a[0], packet_out.PacketOut):
-                        resp_payload = strng.to_text(a[0].Payload)
+                        resp_payload = strng.to_text(a[0].outpacket.Payload)
                     else:
                         resp_payload = strng.to_text(a[0][0].Payload)
                     if resp_payload.startswith('mismatch:'):
@@ -1443,7 +1434,14 @@ class GroupMember(automat.Automat):
             if _Debug:
                 lg.args(_DebugLevel, args=err.value.args)
             try:
-                resp_payload = strng.to_text(err.value.args[0].Payload)
+                if isinstance(err.value.args[0], str):
+                    resp_payload = strng.to_text(err.value.args[0])
+                elif isinstance(err.value.args[0], packet_out.PacketOut):
+                    resp_payload = strng.to_text(err.value.args[0].outpacket.Payload)
+                elif isinstance(err.value.args[0], signed.Packet):
+                    resp_payload = strng.to_text(err.value.args[0].Payload)
+                else:
+                    resp_payload = strng.to_text(err.value.args[0][0].Payload)
                 if _Debug:
                     lg.dbg(_DebugLevel, resp_payload)
                 if resp_payload.startswith('identity:'):
