@@ -390,7 +390,12 @@ class Automat(object):
         self._timers = {}
         self._state_callbacks = {}
         self._callbacks_before_die = {}
-        self.init(**kwargs)
+        try:
+            self.init(**kwargs)
+        except Exception as exc:
+            self.exc(msg='Exception in {}:{} automat init(), state is {}: {}'.format(
+                self.id, self.name, self.state, exc))
+            raise exc
         self.startTimers()
         self.register()
         self.publish_events = publish_events
@@ -406,6 +411,7 @@ class Automat(object):
         """
         global _StateChangedCallback
         global _GlobalLogTransitions
+        global _LogFile
         if self is None:
             self.log(self.debug_level, 'Some crazy stuff happens?')
             return
@@ -418,7 +424,7 @@ class Automat(object):
         if erase_index:
             erase_index(automatid)
         if _GlobalLogTransitions or self.log_transitions:
-            if self.log:
+            if _LogFile:
                 self.log(debug_level, 'DESTROYED AUTOMAT with index %d, total running %d' % (
                     index, len(objects())))
 
@@ -624,7 +630,14 @@ class Automat(object):
         """
         return self._timers
 
-    def to_json(self):
+    def to_json(self, short=True):
+        if short:
+            return {
+                'index': self.index,
+                'id': self.id,
+                'name': self.__class__.__name__,
+                'state': self.state,
+            }
         return {
             'index': self.index,
             'id': self.id,
@@ -697,8 +710,11 @@ class Automat(object):
                 else:
                     if not isinstance(s, unicode):  # @UndefinedVariable
                         s = s.decode('utf-8')
-                _LogFile.write(s)
-                _LogFile.flush()
+                try:
+                    _LogFile.write(s)
+                    _LogFile.flush()
+                except:
+                    pass
                 _LogsCount += 1
 
     def addStateChangedCallback(self, cb, oldstate=None, newstate=None, callback_id=None):
