@@ -38,7 +38,7 @@ from __future__ import print_function
 #------------------------------------------------------------------------------
 
 _Debug = False
-_DebugLevel = 6
+_DebugLevel = 10
 
 #------------------------------------------------------------------------------
 
@@ -61,9 +61,16 @@ from lib import utime
 from lib import strng
 
 from main import settings
+from main import listeners
 
 from crypt import my_keys
 
+from access import group_member
+
+from p2p import online_status
+
+from userid import global_id
+from userid import id_url
 from userid import my_id
 
 #------------------------------------------------------------------------------
@@ -90,8 +97,6 @@ MESSAGE_TYPE_CODES = {
 #------------------------------------------------------------------------------
 
 def init(filepath=None):
-    """
-    """
     global _HistoryDB
     global _HistoryCursor
 
@@ -151,8 +156,6 @@ def init(filepath=None):
 
 
 def shutdown():
-    """
-    """
     global _HistoryDB
     global _HistoryCursor
     if _Debug:
@@ -184,9 +187,23 @@ def convert_json(blob):
 
 #------------------------------------------------------------------------------
 
-def build_json_message(data, message_id, message_time=None, sender=None, recipient=None, message_type=None, direction=None):
-    """
-    """
+def get_conversation_id(sender_local_key_id, recipient_local_key_id, payload_type, ):
+    conversation_id = None
+    if payload_type in [3, 4, ]:
+        conversation_id = '{}&{}'.format(recipient_local_key_id, recipient_local_key_id)
+    elif payload_type == 2:
+        if recipient_local_key_id < sender_local_key_id:
+            conversation_id = '{}&{}'.format(recipient_local_key_id, sender_local_key_id)
+        else:
+            conversation_id = '{}&{}'.format(sender_local_key_id, recipient_local_key_id)
+    else:
+        lg.err('unexpected message type: %r' % payload_type)
+        return None
+    return conversation_id
+
+#------------------------------------------------------------------------------
+
+def build_json_message(data, message_id, message_time=None, sender=None, recipient=None, message_type=None, direction=None, conversation_id=None):
     if not sender:
         sender = my_id.getGlobalID(key_alias='master')
     if not recipient:
@@ -212,37 +229,110 @@ def build_json_message(data, message_id, message_time=None, sender=None, recipie
             'glob_id': recipient,
         },
         'direction': direction,
+        'conversation_id': conversation_id,
     }
     return new_json
 
+
+def build_json_conversation(**record):
+    conv = {
+        'key_id': '',
+        'label': '',
+        'state': 'OFFLINE',
+        'index': None,
+        'id': None,
+        'name': None,
+        'repr': None,
+        'events': None,
+    }
+    conv.update(record)
+    if conv['type'] == 'private_message':
+        local_key_id1, _, local_key_id2 = conv['conversation_id'].partition('&')
+        try:
+            local_key_id1 = int(local_key_id1)
+            local_key_id2 = int(local_key_id2)
+        except:
+            lg.exc()
+            return None
+        usr1 = my_keys.get_local_key(local_key_id1)
+        usr2 = my_keys.get_local_key(local_key_id2)
+        if not usr1 or not usr2:
+            # lg.warn('%r %r : not found sender or recipient key_id for %r' % (usr1, usr2, conv, ))
+            return None
+        usr1 = usr1.replace('master$', '')
+        usr2 = usr2.replace('master$', '')
+        idurl1 = global_id.glob2idurl(usr1, as_field=True)
+        idurl2 = global_id.glob2idurl(usr2, as_field=True)
+        conv_key_id = None
+        conv_label = None
+        user_idurl = None
+        if (id_url.is_cached(idurl1) and idurl1 == my_id.getIDURL()) or usr1.split('@')[0] == my_id.getIDName():
+            user_idurl = idurl2
+            conv_key_id = global_id.UrlToGlobalID(idurl2, include_key=True)
+            conv_label = conv_key_id.replace('master$', '').split('@')[0]
+        if (id_url.is_cached(idurl2) and idurl2 == my_id.getIDURL()) or usr2.split('@')[0] == my_id.getIDName():
+            user_idurl = idurl1
+            conv_key_id = global_id.UrlToGlobalID(idurl1, include_key=True)
+            conv_label = conv_key_id.replace('master$', '').split('@')[0]
+        if conv_key_id:
+            conv['key_id'] = conv_key_id
+        if conv_label:
+            conv['label'] = conv_label
+        else:
+            conv['label'] = conv_key_id
+        if user_idurl:
+            on_st = online_status.getInstance(user_idurl, autocreate=False)
+            if on_st:
+                conv.update(on_st.to_json())
+    elif conv['type'] == 'group_message' or conv['type'] == 'personal_message':
+        local_key_id, _, _ = conv['conversation_id'].partition('&')
+        try:
+            local_key_id = int(local_key_id)
+        except:
+            lg.exc()
+            return None
+        key_id = my_keys.get_local_key(local_key_id)
+        if not key_id:
+            # lg.warn('key_id was not found for %r' % conv)
+            return None
+        conv['key_id'] = key_id
+        conv['label'] = my_keys.get_label(key_id) or key_id
+        gm = group_member.get_active_group_member(key_id)
+        if gm:
+            conv.update(gm.to_json())
+    return conv
+
 #------------------------------------------------------------------------------
 
-def insert_message(message_json):
+def insert_message(data, message_id, message_time=None, sender=None, recipient=None, message_type=None, direction=None):
     """
     Writes JSON message to the message database.
     """
-    try:
-        sender_glob_id = message_json['sender']['glob_id']
-        recipient_glob_id = message_json['recipient']['glob_id']
-        direction = message_json['direction']
-        payload_type = MESSAGE_TYPES.get(message_json['payload']['msg_type'], 1)
-        payload_time = message_json['payload']['time']
-        payload_message_id = message_json['payload']['message_id']
-        payload_body =  message_json['payload']['data']
-    except:
-        lg.exc()
-        return False
+    payload_time = message_time or utime.utcnow_to_sec1970()
+    payload_message_id = strng.to_text(message_id)
+    payload_type = MESSAGE_TYPES.get(message_type, 1)
+    if not sender:
+        sender = my_id.getGlobalID(key_alias='master')
+    if not recipient:
+        recipient = my_id.getGlobalID(key_alias='master')
+    if direction is None:
+        if message_type in ['private_message', None, ]:
+            direction = 'out' if sender == my_id.getGlobalID(key_alias='master') else 'in'
+        else:
+            direction = 'in'
+    else:
+        direction = direction.replace('incoming', 'in').replace('outgoing', 'out')
     if _Debug:
-        lg.args(_DebugLevel, sender=sender_glob_id, recipient=recipient_glob_id, typ=payload_type, message_id=payload_message_id)
-    recipient_local_key_id = my_keys.get_local_key_id(recipient_glob_id)
+        lg.args(_DebugLevel, sender=sender, recipient=recipient, typ=payload_type, dir=direction, message_id=payload_message_id)
+    recipient_local_key_id = my_keys.get_local_key_id(recipient)
     if payload_type in [3, 4, ]:
         sender_local_key_id = recipient_local_key_id
     else:
-        sender_local_key_id = my_keys.get_local_key_id(sender_glob_id)
+        sender_local_key_id = my_keys.get_local_key_id(sender)
     if sender_local_key_id is None or recipient_local_key_id is None:
         lg.err('failed to store message because local_key_id is not found, sender=%r recipient=%r' % (
             sender_local_key_id, recipient_local_key_id, ))
-        return False
+        return None
     cur().execute('''INSERT INTO history (
             sender_local_key_id,
             sender_id,
@@ -255,35 +345,37 @@ def insert_message(message_json):
             payload_body
         ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)''', (
         sender_local_key_id,
-        sender_glob_id,
+        sender,
         recipient_local_key_id,
-        recipient_glob_id,
+        recipient,
         0 if direction == 'in' else 1,
         payload_type,
         payload_time,
         payload_message_id,
-        payload_body,
+        data,
     ))
     db().commit()
-    update_conversation(sender_local_key_id, recipient_local_key_id, payload_type, payload_time, payload_message_id)
-    return True
+    conversation_id = update_conversation(sender_local_key_id, recipient_local_key_id, payload_type, payload_time, payload_message_id)
+    snap_id = '{}/{}'.format(conversation_id, payload_message_id)
+    message_json = build_json_message(
+        sender=sender,
+        recipient=recipient,
+        direction=direction,
+        conversation_id=conversation_id,
+        message_type=MESSAGE_TYPE_CODES.get(int(payload_type), 'private_message'),
+        message_time=payload_time,
+        message_id=payload_message_id,
+        data=data,
+    )
+    listeners.push_snapshot('message', snap_id=snap_id, created=payload_time, data=message_json)
+    return message_json
 
 
 def update_conversation(sender_local_key_id, recipient_local_key_id, payload_type, payload_time, payload_message_id):
-    conversation_id = None
-    if payload_type in [3, 4, ]:
-        conversation_id = '{}&{}'.format(recipient_local_key_id, recipient_local_key_id)
-    elif payload_type == 2:
-        if recipient_local_key_id < sender_local_key_id:
-            conversation_id = '{}&{}'.format(recipient_local_key_id, sender_local_key_id)
-        else:
-            conversation_id = '{}&{}'.format(sender_local_key_id, recipient_local_key_id)
-    else:
-        lg.err('unexpected message type: %r' % payload_type)
-        return
+    conversation_id = get_conversation_id(sender_local_key_id, recipient_local_key_id, payload_type)
     if conversation_id is None:
         lg.err('failed to update conversation, local_key_id was not found')
-        return
+        return None
     sql = 'SELECT * FROM conversations WHERE conversation_id=?'
     params = [conversation_id, ]
     found_conversation = list(cur().execute(sql, params))
@@ -297,12 +389,18 @@ def update_conversation(sender_local_key_id, recipient_local_key_id, payload_typ
         lg.args(_DebugLevel, conversation_id=conversation_id, found_conversation=len(found_conversation), params=params)
     cur().execute(sql, params)
     db().commit()
-    return True
+    if not found_conversation:
+        listeners.push_snapshot('conversation', snap_id=conversation_id, data=build_json_conversation(
+            conversation_id=conversation_id,
+            type=MESSAGE_TYPE_CODES.get(int(payload_type), 'private_message'),
+            started=payload_time,
+            last_updated=payload_time,
+            last_message_id=payload_message_id,
+        ))
+    return conversation_id
 
 
-def query_messages(sender_id=None, recipient_id=None, bidirectional=True, order_by_time=True, message_types=[], offset=None, limit=None):
-    """
-    """
+def query_messages(sender_id=None, recipient_id=None, bidirectional=True, order_by_time=True, message_types=[], offset=None, limit=None, raw_results=False):
     sql = 'SELECT * FROM history'
     q = ''
     params = []
@@ -359,12 +457,19 @@ def query_messages(sender_id=None, recipient_id=None, bidirectional=True, order_
         if recipient_local_key_id not in local_key_ids:
             local_key_ids[recipient_local_key_id] = my_keys.get_local_key(recipient_local_key_id)
         if not local_key_ids.get(sender_local_key_id) or not local_key_ids.get(recipient_local_key_id):
-            lg.warn('unknown sender or recipient local key_id')
+            # lg.warn('unknown sender or recipient local key_id')
+            continue
+        if raw_results:
+            if order_by_time:
+                results.insert(0, row)
+            else:
+                results.append(row)
             continue
         json_msg = build_json_message(
             sender=sender_id_recorded,
             recipient=recipient_id_recorded,
             direction='in' if row[4] == 0 else 'out',
+            conversation_id=get_conversation_id(sender_local_key_id, recipient_local_key_id, int(row[5])),
             message_type=MESSAGE_TYPE_CODES.get(int(row[5]), 'private_message'),
             message_time=row[6],
             message_id=row[7],
@@ -400,7 +505,7 @@ def list_conversations(order_by_time=True, message_types=[], offset=None, limit=
     for row in cur().execute(sql, params):
         results.append(dict(
             conversation_id=row[0],
-            type=MESSAGE_TYPE_CODES.get(int(row[1]), 'private_message'), 
+            type=MESSAGE_TYPE_CODES.get(int(row[1]), 'private_message'),
             started=row[2],
             last_updated=row[3],
             last_message_id=row[4],
@@ -547,6 +652,85 @@ def check_create_rename_key(new_public_key, new_key_id, new_local_key_id):
 
 #------------------------------------------------------------------------------
 
+def fetch_conversations(order_by_time=True, message_types=[], offset=None, limit=None):
+    conversations = []
+    for conv_record in list(list_conversations(
+        order_by_time=order_by_time,
+        message_types=message_types,
+        offset=offset,
+        limit=limit,
+    )):
+        conv = build_json_conversation(**conv_record)
+        if conv:
+            if conv['key_id']:
+                conversations.append(conv)
+            else:
+                lg.warn('unknown key_id for %r' % conv)
+    return conversations
+
+
+def populate_conversations(message_types=[], offset=0, limit=100, order_by_time=True):
+    for conv in fetch_conversations(
+        order_by_time=order_by_time,
+        message_types=message_types,
+        offset=offset,
+        limit=limit,
+    ):
+        listeners.push_snapshot('conversation', snap_id=conv['conversation_id'], data=conv)
+
+
+def populate_messages(recipient_id=None, sender_id=None, message_types=[], offset=0, limit=100):
+    if recipient_id:
+        if not recipient_id.count('@'):
+            from contacts import contactsdb
+            recipient_idurl = contactsdb.find_correspondent_by_nickname(recipient_id)
+            if not recipient_idurl:
+                lg.err('recipient %r was not found' % recipient_id)
+                return
+            recipient_id = global_id.UrlToGlobalID(recipient_idurl)
+        recipient_glob_id = global_id.ParseGlobalID(recipient_id)
+        if not recipient_glob_id['idurl']:
+            lg.err('wrong recipient_id')
+            return
+        recipient_id = global_id.MakeGlobalID(**recipient_glob_id)
+        if not my_keys.is_valid_key_id(recipient_id):
+            lg.err('invalid recipient_id: %s' % recipient_id)
+            return
+    if sender_id:
+        sender_local_key_id = my_keys.get_local_key_id(sender_id)
+        if sender_local_key_id is None:
+            return
+    if recipient_id:
+        recipient_local_key_id = my_keys.get_local_key_id(recipient_id)
+        if recipient_local_key_id is None:
+            lg.warn('recipient %r local key id was not registered' % recipient_id)
+            return
+    for row in query_messages(
+        sender_id=sender_id,
+        recipient_id=recipient_id,
+        bidirectional=False,
+        message_types=message_types,
+        offset=offset,
+        limit=limit,
+        raw_results=True,
+    ):
+        conversation_id = get_conversation_id(row[0], row[2], int(row[5]))
+        if conversation_id is None:
+            continue
+        snap_id = '{}/{}'.format(conversation_id, row[7])
+        listeners.push_snapshot('message', snap_id=snap_id, created=row[6], data=build_json_message(
+            sender=row[1],
+            recipient=row[3],
+            direction='in' if row[4] == 0 else 'out',
+            conversation_id=conversation_id,
+            message_type=MESSAGE_TYPE_CODES.get(int(row[5]), 'private_message'),
+            message_time=row[6],
+            message_id=row[7],
+            data=json.loads(row[8]),
+        ))
+
+#------------------------------------------------------------------------------
+
 def main():
     import pprint
     my_keys.init()
