@@ -57,6 +57,7 @@ from system import bpio
 from system import local_fs
 
 from main import settings
+from main import listeners
 
 from services import driver
 
@@ -106,8 +107,6 @@ def init():
 
 
 def shutdown():
-    """
-    """
     global _SuppliersChangedCallback
     global _CustomersChangedCallback
     if _Debug:
@@ -148,8 +147,6 @@ def supplier(index, customer_idurl=None):
 
 
 def all_suppliers(as_dict=False):
-    """
-    """
     global _SuppliersList
     if as_dict:
         return _SuppliersList
@@ -232,8 +229,6 @@ def add_supplier(idurl, position=None, customer_idurl=None):
 
 
 def erase_supplier(idurl=None, position=None, customer_idurl=None):
-    """
-    """
     global _SuppliersList
     if not customer_idurl:
         customer_idurl = my_id.getIDURL()
@@ -412,14 +407,6 @@ def set_correspondents(idlist):
     _CorrespondentsList = [(id_url.field(idurl_name[0]).to_bin(), idurl_name[1]) for idurl_name in idlist]
 
 
-def clear_correspondents():
-    """
-    Remove all correspondents without notification.
-    """
-    global _CorrespondentsList
-    _CorrespondentsList = []
-
-
 def add_correspondent(idurl, nickname=''):
     """
     Add correspondent, execute notification callback and return its position in
@@ -432,6 +419,10 @@ def add_correspondent(idurl, nickname=''):
     _CorrespondentsList.append((idurl.to_bin(), nickname, ))
     if _CorrespondentsChangedCallback is not None:
         _CorrespondentsChangedCallback(curlist, _CorrespondentsList)
+    listeners.push_snapshot('correspondent', snap_id=idurl.to_bin(), data=dict(
+        idurl=idurl.to_bin(),
+        nickname=nickname,
+    ))
     return len(curlist)
 
 
@@ -449,20 +440,20 @@ def remove_correspondent(idurl):
             _CorrespondentsList.remove(tupl)
             if _CorrespondentsChangedCallback is not None:
                 _CorrespondentsChangedCallback(curlist, _CorrespondentsList)
+            listeners.push_snapshot('correspondent', snap_id=idurl.to_bin(), deleted=True, data=dict(
+                idurl=idurl.to_bin(),
+                nickname=tupl[1],
+            ))
             return True
     return False
 
 
-def update_correspondents(idslist):
-    """
-    Set correspondents ID's list, input items are tuples: (idurl, nickname, ).
-    """
-    global _CorrespondentsChangedCallback
-    oldcorrespondents = list(correspondents())
-    idslist = list(filter(lambda i: id_url.is_cached(i[0]), idslist))
-    set_correspondents(idslist)
-    if _CorrespondentsChangedCallback is not None:
-        _CorrespondentsChangedCallback(oldcorrespondents, correspondents())
+def populate_correspondents():
+    for corr in correspondents():
+        listeners.push_snapshot('correspondent', snap_id=corr[0], data=dict(
+            idurl=corr[0],
+            nickname=corr[1],
+        ))
 
 #-------------------------------------------------------------------------------
 
@@ -509,8 +500,6 @@ def num_suppliers(customer_idurl=None):
 
 
 def total_suppliers():
-    """
-    """
     global _SuppliersList
     result = set()
     for suppliers_list in _SuppliersList.values():
@@ -820,6 +809,9 @@ def load_contacts():
     if _CorrespondentsChangedCallback is not None:
         _CorrespondentsChangedCallback([], correspondents())
     AddContactsChangedCallback(on_contacts_changed)
+    if listeners.is_populate_requered('correspondent'):
+        listeners.populate_later().remove('correspondent')
+        populate_correspondents()
 
 #------------------------------------------------------------------------------
 
@@ -880,8 +872,6 @@ def get_correspondent_identity(idurl):
 
 
 def get_correspondent_nickname(correspondent_idurl):
-    """
-    """
     for idurl, nickname in correspondents():
         if id_url.field(idurl).to_bin() == id_url.field(correspondent_idurl).to_bin():
             return nickname
@@ -906,15 +896,11 @@ def on_contacts_changed(old_contacts_list, new_contacts_list):
 #------------------------------------------------------------------------------
 
 def read_customers_meta_info_all():
-    """
-    """
     global _CustomersMetaInfo
     return _CustomersMetaInfo
 
 
 def write_customers_meta_info_all(new_customers_info):
-    """
-    """
     global _CustomersMetaInfo
     _CustomersMetaInfo = new_customers_info
     json_info = {k: jsn.dict_keys_to_text(v) for k, v in id_url.to_bin_dict(_CustomersMetaInfo).items()}
@@ -930,8 +916,6 @@ def write_customers_meta_info_all(new_customers_info):
 
 
 def add_customer_meta_info(customer_idurl, info):
-    """
-    """
     global _CustomersMetaInfo
     customer_idurl = id_url.field(customer_idurl)
     if not customer_idurl.is_latest():
@@ -968,8 +952,6 @@ def add_customer_meta_info(customer_idurl, info):
 
 
 def remove_customer_meta_info(customer_idurl):
-    """
-    """
     global _CustomersMetaInfo
     customer_idurl = id_url.field(customer_idurl)
     if not customer_idurl.is_latest():
@@ -993,8 +975,6 @@ def remove_customer_meta_info(customer_idurl):
 
 
 def get_customer_meta_info(customer_idurl):
-    """
-    """
     global _CustomersMetaInfo
     customer_idurl = id_url.field(customer_idurl)
     if not customer_idurl.is_latest():
@@ -1009,8 +989,6 @@ def get_customer_meta_info(customer_idurl):
 #------------------------------------------------------------------------------
 
 def add_supplier_meta_info(supplier_idurl, info, customer_idurl=None):
-    """
-    """
     global _SuppliersMetaInfo
     if not customer_idurl:
         customer_idurl = my_id.getIDURL()
@@ -1027,8 +1005,6 @@ def add_supplier_meta_info(supplier_idurl, info, customer_idurl=None):
 
 
 def remove_supplier_meta_info(supplier_idurl, customer_idurl=None):
-    """
-    """
     global _SuppliersMetaInfo
     if not customer_idurl:
         customer_idurl = my_id.getIDURL()
@@ -1045,8 +1021,6 @@ def remove_supplier_meta_info(supplier_idurl, customer_idurl=None):
 
 
 def get_supplier_meta_info(supplier_idurl, customer_idurl=None):
-    """
-    """
     global _SuppliersMetaInfo
     if not customer_idurl:
         customer_idurl = my_id.getIDURL()
