@@ -65,6 +65,7 @@ from system import local_fs
 
 from main import settings
 from main import events
+from main import listeners
 
 from crypt import key
 from crypt import rsa_key
@@ -84,16 +85,12 @@ _LocalKeysIndex = {}
 #------------------------------------------------------------------------------
 
 def init():
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'my_keys.init')
     scan_local_keys()
 
 
 def shutdown():
-    """
-    """
     global _LatestLocalKeyID
     if _Debug:
         lg.out(_DebugLevel, 'my_keys.shutdown')
@@ -172,8 +169,6 @@ def is_key_registered(key_id, include_master=True):
 
 
 def is_key_private(key_id, include_master=True):
-    """
-    """
     if not is_key_registered(key_id):
         return False
     if include_master:
@@ -237,8 +232,6 @@ def split_key_id(key_id, as_field=True):
 
 
 def is_valid_key_id(global_key_id):
-    """
-    """
     parts = global_id.ParseGlobalID(global_key_id)
     if not parts['key_alias']:
         lg.warn('no key_alias found in the input')
@@ -282,8 +275,6 @@ def get_creator_idurl(key_id, as_field=True):
 #------------------------------------------------------------------------------
 
 def scan_local_keys(keys_folder=None):
-    """
-    """
     global _LatestLocalKeyID
     if not keys_folder:
         keys_folder = settings.KeyStoreDir()
@@ -336,8 +327,6 @@ def scan_local_keys(keys_folder=None):
 
 
 def read_key_file(key_id, keys_folder=None):
-    """
-    """
     if not keys_folder:
         keys_folder = settings.KeyStoreDir()
     key_filepath = os.path.join(keys_folder, '%s.private' % key_id)
@@ -360,6 +349,7 @@ def read_key_file(key_id, keys_folder=None):
                 'body': key_raw_strip,
                 'local_key_id': None,
                 'need_to_convert': True,
+                'active': True,
             }
     except:
         lg.exc()
@@ -368,8 +358,6 @@ def read_key_file(key_id, keys_folder=None):
 
 
 def load_key(key_id, keys_folder=None):
-    """
-    """
     global _LatestLocalKeyID
     if not is_valid_key_id(key_id):
         lg.warn('key is not valid: %r' % key_id)
@@ -397,16 +385,25 @@ def load_key(key_id, keys_folder=None):
             local_keys()[key_object.local_key_id] = key_id
             local_keys_index()[key_object.toPublicString()] = key_object.local_key_id
             if _Debug:
-                lg.out(_DebugLevel, 'my_keys.load_key %r  label=%r  is_private=%r  local_key_id=%r  from %s' % (
-                    key_id, key_object.label, not key_object.isPublic(), key_object.local_key_id, keys_folder, ))
+                lg.out(_DebugLevel, 'my_keys.load_key %r  label=%r  active=%r  is_private=%r  local_key_id=%r  from %s' % (
+                    key_id, key_object.label, key_object.active, not key_object.isPublic(), key_object.local_key_id, keys_folder, ))
         else:
             lg.warn('for key %r local_key_id was not set' % key_id)
+    events.send('key-loaded', data=dict(key_id=key_id, label=key_object.label, key_size=key_object.size(), ))
+    listeners.push_snapshot('key', snap_id=key_id, data=make_key_info(
+        key_object=key_object,
+        key_id=key_id,
+        event='key-loaded',
+        include_private=False,
+        include_local_id=True,
+        include_signature=True,
+        include_label=True,
+        include_state=True,
+    ))
     return True
 
 
 def save_key(key_id, keys_folder=None):
-    """
-    """
     key_object = known_keys()[key_id]
     if key_object is None:
         lg.warn('can not save key %s because it is not loaded yet' % key_id)
@@ -434,8 +431,6 @@ def save_key(key_id, keys_folder=None):
 
 
 def save_keys_local(keys_folder=None):
-    """
-    """
     if not keys_folder:
         keys_folder = settings.KeyStoreDir()
     if _Debug:
@@ -450,8 +445,6 @@ def save_keys_local(keys_folder=None):
 
 
 def save_latest_local_key_id(keys_folder=None):
-    """
-    """
     global _LatestLocalKeyID
     if not keys_folder:
         keys_folder = settings.KeyStoreDir()
@@ -463,9 +456,7 @@ def save_latest_local_key_id(keys_folder=None):
 
 #------------------------------------------------------------------------------
 
-def generate_key(key_id, label='', key_size=4096, keys_folder=None):
-    """
-    """
+def generate_key(key_id, label='', active=True, key_size=4096, keys_folder=None):
     global _LatestLocalKeyID
     key_id = latest_key_id(key_id)
     if is_key_registered(key_id):
@@ -480,6 +471,7 @@ def generate_key(key_id, label='', key_size=4096, keys_folder=None):
     key_object = rsa_key.RSAKey()
     key_object.generate(key_size)
     key_object.label = label
+    key_object.active = active
     key_object.local_key_id = _LatestLocalKeyID
     known_keys()[key_id] = key_object
     if _Debug:
@@ -488,12 +480,20 @@ def generate_key(key_id, label='', key_size=4096, keys_folder=None):
         keys_folder = settings.KeyStoreDir()
     save_key(key_id, keys_folder=keys_folder)
     events.send('key-generated', data=dict(key_id=key_id, label=label, key_size=key_size, ))
+    listeners.push_snapshot('key', snap_id=key_id, data=make_key_info(
+        key_object=key_object,
+        key_id=key_id,
+        event='key-generated',
+        include_private=False,
+        include_local_id=True,
+        include_signature=True,
+        include_label=True,
+        include_state=True,
+    ))
     return key_object
 
 
-def register_key(key_id, key_object_or_string, label='', keys_folder=None):
-    """
-    """
+def register_key(key_id, key_object_or_string, label='', active=True, keys_folder=None):
     global _LatestLocalKeyID
     key_id = latest_key_id(key_id)
     if is_key_registered(key_id):
@@ -502,7 +502,7 @@ def register_key(key_id, key_object_or_string, label='', keys_folder=None):
     if not keys_folder:
         keys_folder = settings.KeyStoreDir()
     if not label:
-        label = 'key%s' % utime.make_timestamp() 
+        label = 'key%s' % utime.make_timestamp()
     if strng.is_string(key_object_or_string):
         key_object_or_string = strng.to_bin(key_object_or_string)
         if _Debug:
@@ -516,6 +516,7 @@ def register_key(key_id, key_object_or_string, label='', keys_folder=None):
         if _Debug:
             lg.out(_DebugLevel, 'my_keys.register_key %r from object' % key_id)
         key_object = key_object_or_string
+        label = key_object.label or label
     known_local_key_id = local_keys_index().get(key_object.toPublicString())
     if known_local_key_id is not None:
         known_key_id = local_keys().get(known_local_key_id)
@@ -530,17 +531,27 @@ def register_key(key_id, key_object_or_string, label='', keys_folder=None):
         save_latest_local_key_id(keys_folder=keys_folder)
         new_local_key_id = _LatestLocalKeyID
     key_object.local_key_id = new_local_key_id
+    key_object.label = label
+    key_object.active = active
     known_keys()[key_id] = key_object
     if _Debug:
         lg.out(_DebugLevel, '    key %r registered' % key_id)
     save_key(key_id, keys_folder=keys_folder)
     events.send('key-registered', data=dict(key_id=key_id, label=label, key_size=key_object.size(), ))
+    listeners.push_snapshot('key', snap_id=key_id, data=make_key_info(
+        key_object=key_object,
+        key_id=key_id,
+        event='key-registered',
+        include_private=False,
+        include_local_id=True,
+        include_signature=True,
+        include_label=True,
+        include_state=True,
+    ))
     return key_object
 
 
 def erase_key(key_id, keys_folder=None):
-    """
-    """
     key_id = latest_key_id(key_id)
     if not is_key_registered(key_id):
         lg.warn('key %s is not registered' % key_id)
@@ -565,12 +576,20 @@ def erase_key(key_id, keys_folder=None):
     if _Debug:
         lg.out(_DebugLevel, '    key %s removed, file %s deleted' % (key_id, key_filepath))
     events.send('key-erased', data=dict(key_id=key_id, is_private=is_private))
+    listeners.push_snapshot('key', snap_id=key_id, deleted=True, data=make_key_info(
+        key_object=None,
+        key_id=key_id,
+        event='key-erased',
+        include_private=False,
+        include_local_id=True,
+        include_signature=True,
+        include_label=True,
+        include_state=True,
+    ))
     return True
 
 
 def validate_key(key_object):
-    """
-    """
     sample_data = strng.to_bin(base64.b64encode(os.urandom(256)))
     sample_hash_base = hashes.sha1(sample_data, hexdigest=True)
     sample_signature = key_object.sign(sample_hash_base)
@@ -586,8 +605,6 @@ def validate_key(key_object):
 
 
 def rename_key(current_key_id, new_key_id, keys_folder=None):
-    """
-    """
     if not keys_folder:
         keys_folder = settings.KeyStoreDir()
     if current_key_id not in known_keys():
@@ -651,6 +668,16 @@ def sign_key(key_id, keys_folder=None, ignore_shared_keys=False, save=True):
     if save:
         save_key(key_id, keys_folder=keys_folder)
     events.send('key-signed', data=dict(key_id=key_id, label=key_object.label, key_size=key_object.size(), ))
+    listeners.push_snapshot('key', snap_id=key_id, data=make_key_info(
+        key_object=key_object,
+        key_id=key_id,
+        event='key-signed',
+        include_private=False,
+        include_local_id=True,
+        include_signature=True,
+        include_label=True,
+        include_state=True,
+    ))
     return key_object
 
 #------------------------------------------------------------------------------
@@ -764,8 +791,6 @@ def decrypt(key_id, inp):
 #------------------------------------------------------------------------------
 
 def serialize_key(key_id):
-    """
-    """
     key_id = latest_key_id(key_id)
     if not is_key_registered(key_id):
         raise Exception('key %s is not registered' % key_id)
@@ -777,8 +802,6 @@ def serialize_key(key_id):
 
 
 def unserialize_key_to_object(raw_string):
-    """
-    """
     try:
         key_object = rsa_key.RSAKey()
         key_object.fromString(raw_string)
@@ -807,7 +830,7 @@ def get_label(key_id):
     """
     key_id = strng.to_text(key_id)
     if not is_key_registered(key_id):
-        return None
+        return ''
     return key_obj(key_id).label
 
 
@@ -831,11 +854,30 @@ def get_local_key(local_key_id):
 
 #------------------------------------------------------------------------------
 
+def is_active(key_id):
+    """
+    Returns True if given key has "active" state. If key is not set to "active" state, certain parts of the software should not use it.
+    """
+    key_id = strng.to_text(key_id)
+    if not is_key_registered(key_id):
+        return None
+    return key_obj(key_id).active
+
+
+def set_active(key_id, active=True):
+    key_id = strng.to_text(key_id)
+    if not is_key_registered(key_id):
+        return
+    key_obj(key_id).active = active
+
+#------------------------------------------------------------------------------
+
 def make_master_key_info(include_private=False):
     r = {
         'key_id': my_id.getGlobalID(key_alias='master'),
         'alias': 'master',
         'label': my_id.getGlobalID(key_alias='master'),
+        'active': True,
         'creator': my_id.getIDURL(),
         'is_public': key.MyPrivateKeyObject().isPublic(),
         # 'fingerprint': str(key.MyPrivateKeyObject().fingerprint()),
@@ -856,7 +898,7 @@ def make_master_key_info(include_private=False):
 
 def make_key_info(key_object, key_id=None, key_alias=None, creator_idurl=None,
                   include_private=False, generate_signature=False, include_signature=False,
-                  include_local_id=False, include_label=True, ):
+                  include_local_id=False, include_label=True, include_state=False, event=None):
     if key_id:
         key_id = latest_key_id(key_id)
         key_alias, creator_idurl = split_key_id(key_id)
@@ -866,36 +908,40 @@ def make_key_info(key_object, key_id=None, key_alias=None, creator_idurl=None,
         'key_id': key_id,
         'alias': key_alias,
         'creator': creator_idurl,
-        'public': strng.to_text(key_object.toPublicString()),
+        'public': strng.to_text(key_object.toPublicString()) if key_object else None,
         'private': None,
         'include_private': include_private,
     }
+    if event:
+        r['event'] = event
     if include_label:
-        r['label'] = key_object.label
-    if key_object.isPublic():
+        r['label'] = key_object.label if key_object else ''
+    if include_state:
+        r['active'] = key_object.active if key_object else True
+    if key_object and key_object.isPublic():
         r['is_public'] = True
         if include_private:
             raise Exception('this key contains only public component')
     else:
         r['is_public'] = not include_private
         if include_private:
-            r['private'] = strng.to_text(key_object.toPrivateString())
-    if hasattr(key_object, 'size'):
+            r['private'] = strng.to_text(key_object.toPrivateString()) if key_object else None
+    if key_object and hasattr(key_object, 'size'):
         r['size'] = strng.to_text(key_object.size())
     else:
         r['size'] = '0'
     if include_local_id:
-        r['local_key_id'] = getattr(key_object, 'local_key_id', None)
-    if generate_signature:
+        r['local_key_id'] = getattr(key_object, 'local_key_id', None) if key_object else None
+    if key_object and generate_signature:
         r = sign_key_info(r)
     else:
-        if include_signature and key_object.isSigned():
+        if include_signature and key_object and key_object.isSigned():
             r['signature'] = key_object.signed[0]
             r['signature_pubkey'] = key_object.signed[1]
     return r
 
 
-def get_key_info(key_id, include_private=False, include_signature=False, generate_signature=False, include_label=True):
+def get_key_info(key_id, include_private=False, include_signature=False, generate_signature=False, include_label=True, include_state=False):
     """
     Returns dictionary with full key info or raise an Exception.
     """
@@ -924,6 +970,7 @@ def get_key_info(key_id, include_private=False, include_signature=False, generat
         include_signature=include_signature,
         generate_signature=generate_signature,
         include_label=include_label,
+        include_state=include_state,
     )
     return key_info
 
@@ -940,6 +987,7 @@ def read_key_info(key_json):
         if not key_object:
             raise Exception('unserialize failed')
         key_object.label = strng.to_text(key_json.get('label', ''))
+        key_object.active = key_json.get('active', True)
         if 'signature' in key_json and 'signature_pubkey' in key_json:
             key_object.signed = (key_json['signature'], key_json['signature_pubkey'], )
     except:
@@ -1003,6 +1051,21 @@ def check_rename_my_keys(prefix=None):
 
 #------------------------------------------------------------------------------
 
+def populate_keys():
+    for key_id, key_object in known_keys().items():
+        listeners.push_snapshot('key', snap_id=key_id, data=make_key_info(
+            key_object=key_object,
+            key_id=key_id,
+            event=None,
+            include_private=False,
+            include_local_id=True,
+            include_signature=True,
+            include_label=True,
+            include_state=True,
+        ))
+
+#------------------------------------------------------------------------------
+
 if __name__ == '__main__':
     bpio.init()
     lg.set_debug_level(18)
