@@ -160,21 +160,15 @@ def GetLastFireTime():
 
 
 def ClearLastFireTime():
-    """
-    """
     _LastFireTime = 0
 
 
 def AddSupplierToFire(idurl):
-    """
-    """
     global _SuppliersToFire
     _SuppliersToFire.append(idurl)
 
 
 def IsAllHired():
-    """
-    """
     if settings.getSuppliersNumberDesired() < 0:
         # I must know how many suppliers I want
         lg.warn('my desired number of suppliers not set')
@@ -702,8 +696,8 @@ class FireHire(automat.Automat):
             time.strftime('%d-%m-%Y %H:%M:%S'),
             my_id.getIDURL(),
         )
-        from main import control
-        control.on_suppliers_changed(current_suppliers)
+        # from main import control
+        # control.on_suppliers_changed(current_suppliers)
         if family_position < 0:
             lg.info('added new supplier, family position unknown: %s desired_suppliers=%d current_suppliers=%d' % (
                 new_idurl, desired_suppliers, len(contactsdb.suppliers())))
@@ -765,8 +759,8 @@ class FireHire(automat.Automat):
         current_suppliers = current_suppliers[:desired_suppliers]
         contactsdb.update_suppliers(current_suppliers)
         contactsdb.save_suppliers()
-        from main import control
-        control.on_suppliers_changed(current_suppliers)
+        # from main import control
+        # control.on_suppliers_changed(current_suppliers)
         for position, supplier_idurl in removed_suppliers:
             events.send('supplier-modified', data=dict(
                 new_idurl=None, old_idurl=supplier_idurl, position=position,
