@@ -538,6 +538,7 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             key_alias=data['alias'],
             key_size=int(data['key_size']) if 'key_size' in data else None,
             label=data.get('label', ''),
+            active=bool(data.get('active', '1') in ['1', 'true', ]),
             include_private=bool(data.get('include_private', '0') in ['1', 'true', ]),
         )
 
@@ -551,6 +552,16 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             label=data['label'],
         )
 
+    @POST('^/k/st$')
+    @POST('^/v1/key/state$')
+    @POST('^/key/state/v1$')
+    def key_state_v1(self, request):
+        data = _request_data(request, mandatory_keys=['active', 'key_id', ])
+        return api.key_label(
+            key_id=data['key_id'],
+            active=bool(data.get('active', '1') in ['1', 'true', ]),
+        )
+
     @DELETE('^/k/d$')
     @DELETE('^/v1/key/erase$')
     @DELETE('^/key/erase/v1$')
@@ -659,7 +670,6 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             local_path=data['local_path'],
             remote_path=data['remote_path'],
             wait_result=bool(data.get('wait_result', '0') in ['1', 'true', ]),
-            open_share=bool(data.get('open_share', '0') in ['1', 'true', ]),
             publish_events=bool(data.get('publish_events', '0') in ['1', 'true', ]),
         )
 
@@ -685,7 +695,6 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             remote_path=data['remote_path'],
             destination_path=data.get('destination_folder', None),
             wait_result=bool(data.get('wait_result', '0') in ['1', 'true', ]),
-            open_share=bool(data.get('open_share', '1') in ['1', 'true', ]),
             publish_events=bool(data.get('publish_events', '0') in ['1', 'true', ]),
         )
 
@@ -714,6 +723,12 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             include_granted=bool(_request_arg(request, 'granted', '1') in ['1', 'true', ]),
         )
 
+    @GET('^/sh/i$')
+    @GET('^/v1/share/info$')
+    @GET('^/share/info/v1$')
+    def share_info_v1(self, request):
+        return api.share_info(key_id=_request_arg(request, 'key_id', mandatory=True))
+
     @POST('^/sh/c$')
     @POST('^/v1/share/create$')
     @POST('^/share/create/v1$')
@@ -723,6 +738,7 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             owner_id=data.get('owner_id', None),
             key_size=int(data['key_size']) if 'key_size' in data else None,
             label=data.get('label', ''),
+            active=bool(data.get('active', '1') in ['1', 'true', ]),
         )
 
     @DELETE('^/sh/d$')
@@ -1147,6 +1163,12 @@ class BitDustRESTHTTPServer(JsonAPIResource):
             wait_timeout=_request_data(request).get('wait_timeout', 10),
         )
 
+    @GET('^/svc/h/(?P<service_name>[^/]+)/$')
+    @GET('^/v1/service/health/(?P<service_name>[^/]+)$')
+    @GET('^/service/health/(?P<service_name>[^/]+)/v1$')
+    def service_health_v1(self, request, service_name):
+        return api.service_health(service_name)
+
     #------------------------------------------------------------------------------
 
     @GET('^/pkt/l$')
