@@ -118,8 +118,7 @@ def readSupplierData(supplier_idurl, filename, customer_idurl):
     """
     Read a file from [BitDust data dir]/suppliers/[IDURL] folder.
 
-    The file names right now is ['connected', 'disconnected',
-    'listfiles'].
+    The file names right now is ['connected', 'disconnected', 'listfiles'].
     """
     path = settings.SupplierPath(supplier_idurl, customer_idurl, filename)
     if not os.path.isfile(path):
@@ -161,8 +160,6 @@ def NewBackupID(time_st=None):
 
 
 def TimeStructFromVersion(backupID):
-    """
-    """
     try:
         if backupID.endswith('AM') or backupID.endswith('PM'):
             ampm = backupID[-2:]
@@ -173,7 +170,7 @@ def TimeStructFromVersion(backupID):
             st_time = list(time.strptime(backupID[1:i - 1], '%Y%m%d%I%M%S'))
         if ampm == 'PM':
             st_time[3] += 12
-        return st_time
+        return tuple(st_time)
     except:
         lg.exc()
         return None
@@ -186,6 +183,7 @@ def TimeFromBackupID(backupID):
     try:
         return time.mktime(TimeStructFromVersion(backupID))
     except:
+        lg.exc()
         return None
 
 
@@ -243,7 +241,7 @@ def backup_id_compare(backupID1, backupID2):
     if customerGlobalID1 != customerGlobalID2:
         return cmp(customerGlobalID1, customerGlobalID2)
     return version_compare(version1, version2)
-    
+
 
 def sorted_backup_ids(backupIds, reverse=False):
     """
@@ -262,40 +260,6 @@ def sorted_versions(versions, reverse=False):
 
 #------------------------------------------------------------------------------
 
-
-BASE_ALPHABET = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!()+-#$^&_=@[]{}`~'
-BASE_LIST = BASE_ALPHABET
-BASE_DICT = dict((c, i) for i, c in enumerate(BASE_LIST))
-
-
-def base_decode(string, reverse_base=BASE_DICT):
-    """
-    Not used right now, I was playing with file paths on remote peers.
-    """
-    return base64.decodestring(string)
-#    length = len(reverse_base)
-#    ret = 0
-#    for i, c in enumerate(string[::-1]):
-#        ret += (length ** i) * reverse_base[c]
-#    return ret
-
-
-def base_encode(string, base=BASE_LIST):
-    """
-    Not used at the moment.
-    """
-    return base64.encodestring(string)
-#    length = len(base)
-#    ret = ''
-#    while integer != 0:
-#        ret = base[integer % length] + ret
-#        integer /= length
-#    return ret
-
-
-#------------------------------------------------------------------------------
-
-
 def DigitsOnly(inpt, includes=''):
     """
     Very basic method to convert string to number.
@@ -486,8 +450,6 @@ def RoundupFile(filename, stepsize):
 
 
 def RoundupString(data, stepsize):
-    """
-    """
     size = len(data)
     mod = size % stepsize
     increase = 0
@@ -704,14 +666,11 @@ def percent2string(percent, precis=3):
     A tool to make a string (with % at the end) from given float, ``precis`` is
     precision to round the number.
     """
-    s = float2str(round(percent, precis),
-                  mask=("%%3.%df" % (precis + 2)))
+    s = float2str(round(percent, precis), mask=("%%3.%df" % (precis + 2)))
     return s + '%'
 
 
 def value2percent(value, total, precis=3):
-    """
-    """
     if not total:
         return '0%'
     return percent2string(100.0 * (float(value) / float(total)), precis)
@@ -1055,9 +1014,6 @@ def DoRestart(param='', detach=False, std_out='/dev/null', std_err='/dev/null'):
             # lg.out(2, "misc.DoRestart cmdargs="+str(cmdargs))
             return os.spawnve(os.P_DETACH, starter_filepath, cmdargs, os.environ)  # @UndefinedVariable
 
-        lg.out(2, "misc.DoRestart under Windows param=%s" % param)
-        lg.out(2, "misc.DoRestart sys.executable=" + sys.executable)
-        lg.out(2, "misc.DoRestart sys.argv=" + str(sys.argv))
         pypath = sys.executable
         cmdargs = [sys.executable, ]
         cmdargs.append(sys.argv[0])
@@ -1073,14 +1029,9 @@ def DoRestart(param='', detach=False, std_out='/dev/null', std_err='/dev/null'):
         if detach:
             from system import child_process
             cmdargs = [strng.to_text(a) for a in cmdargs]
-            lg.out(0, 'run : %r' % cmdargs)
             return child_process.detach(cmdargs)
-        lg.out(2, "misc.DoRestart cmdargs=" + str(cmdargs))
         return os.execvpe(pypath, cmdargs, os.environ)
 
-    lg.out(2, "misc.DoRestart under Linux param=%s" % param)
-    lg.out(2, "misc.DoRestart sys.executable=" + sys.executable)
-    lg.out(2, "misc.DoRestart sys.argv=" + str(sys.argv))
     pypyth = sys.executable
     cmdargs = [sys.executable, ]
     if sys.argv[0] == '/usr/share/bitdust/bitdust.py':
@@ -1097,7 +1048,6 @@ def DoRestart(param='', detach=False, std_out='/dev/null', std_err='/dev/null'):
         cmdargs.remove('daemon')
     pid = os.fork()
     if pid != 0:
-        lg.out(2, "misc.DoRestart os.fork returned: " + str(pid))
         return None
     if detach:
         cmdargs[1] = os.path.abspath(cmdargs[1])
@@ -1110,9 +1060,7 @@ def DoRestart(param='', detach=False, std_out='/dev/null', std_err='/dev/null'):
         BITDUST_LOG_USE_COLORS = os.environ.get('BITDUST_LOG_USE_COLORS')
         if BITDUST_LOG_USE_COLORS:
             cmd = 'BITDUST_LOG_USE_COLORS="%s" %s' % (BITDUST_LOG_USE_COLORS, cmd, )
-        lg.out(2, "misc.DoRestart cmd: [%s]" % cmd)
         return os.system(cmd)
-    lg.out(2, "misc.DoRestart cmdargs=" + str(cmdargs))
     return os.execvpe(pypyth, cmdargs, os.environ)
 
 
@@ -1151,8 +1099,9 @@ def LoopAttenuation(current_delay, go_faster, min_delay, max_delay):
     called exactly when 3 seconds passed. But we do not want fixed periods
     sometimes.
 
-    You must be hury when you have a lot of work, in the next moment - need rest.
-    For example - need to read some queue as fast as possible when you have some items inside.
+    It is common to be in hurry when you have a lot of work, but when there is nothing to do you can take rest.
+    For example, while reading a queue of events it make sense to do it as fast as possible, but only when you have some items in the queue.
+    When the queue is empty - you can try to reduce the load on the main process and just check the queue less often.
     This method is used to calculate the delay to the next call of some 'idle' method.
         :param current_delay: current period of time in seconds between calls
         :param go_faster:  if this is True - method should return ``min`` period - call next time as soon as possible
