@@ -215,6 +215,9 @@ def init(UI='', options=None, args=None, overDict=None, executablePath=None):
     from main import events
     events.init()
 
+    from main import listeners
+    listeners.init()
+
     from main import initializer
     IA = initializer.A()
     if _Debug:
@@ -241,6 +244,9 @@ def shutdown():
     from main import shutdowner
     shutdowner.A('reactor-stopped')
 
+    from main import listeners
+    listeners.shutdown()
+
     from main import events
     events.shutdown()
 
@@ -298,8 +304,6 @@ def run_twisted_reactor():
 
 
 def run(UI='', options=None, args=None, overDict=None, executablePath=None, start_reactor=True):
-    """
-    """
     if options and options.cpu_profile:
         import cProfile, pstats, io
         from pstats import SortKey  # @UnresolvedImport
@@ -680,7 +684,7 @@ def main(executable_path=None, start_reactor=True):
 
     if bpio.Android():
         lg.close_intercepted_log_file()
-        lg.open_intercepted_log_file('/storage/emulated/0/.bitdust/logs/android.log')
+        lg.open_intercepted_log_file('/storage/emulated/0/Android/data/org.bitdust_io.bitdust1/files/Documents/.bitdust/logs/android.log')
 
     # sys.excepthook = lg.exception_hook
 
@@ -723,6 +727,10 @@ def main(executable_path=None, start_reactor=True):
     if opts.output:
         logpath = opts.output
     else:
+        try:
+            os.makedirs(os.path.join(appdata, 'logs'), exist_ok=True)
+        except:
+            pass
         logpath = os.path.join(appdata, 'logs', 'stdout.log')
 
     need_redirecting = False
@@ -784,11 +792,10 @@ def main(executable_path=None, start_reactor=True):
             return 0
         from lib import misc
         print_text('new BitDust process will be started in daemon mode\n', nl='')
-        bpio.shutdown()
         result = misc.DoRestart(
             detach=True,
-            std_out=os.path.join(appdata, 'logs', 'stdout.log'),
-            std_err=os.path.join(appdata, 'logs', 'stderr.log'),
+            # std_out=os.path.join(appdata, 'logs', 'stdout.log'),
+            # std_err=os.path.join(appdata, 'logs', 'stderr.log'),
         )
         if result is not None:
             try:
@@ -798,6 +805,7 @@ def main(executable_path=None, start_reactor=True):
                     result = result.pid
                 except:
                     pass
+        bpio.shutdown()
         if opts.coverage:
             cov.stop()
             cov.save()
@@ -841,8 +849,8 @@ def main(executable_path=None, start_reactor=True):
                     misc.DoRestart,
                     param='show' if ui else '',
                     detach=True,
-                    std_out=os.path.join(appdata, 'logs', 'stdout.log'),
-                    std_err=os.path.join(appdata, 'logs', 'stderr.log'),
+                    # std_out=os.path.join(appdata, 'logs', 'stdout.log'),
+                    # std_err=os.path.join(appdata, 'logs', 'stderr.log'),
                 )
                 reactor.stop()  # @UndefinedVariable
             try:
@@ -949,7 +957,7 @@ def main(executable_path=None, start_reactor=True):
                         else:
                             print_text('finished with: %s\n' % x, nl='')
                     else:
-                        print_text('finished successfully\n' % x, nl='')
+                        print_text('finished successfully\n', nl='')
                     reactor.stop()  # @UndefinedVariable
                     bpio.shutdown()
 
