@@ -66,8 +66,6 @@ _Config = None
 
 
 def init(configDir):
-    """
-    """
     global _Config
     if _Config is None:
         _Config = DetailedConfig(configDir)
@@ -78,8 +76,6 @@ def init(configDir):
 
 
 def shutdown():
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'config.shutdown')
     global _Config
@@ -90,8 +86,6 @@ def shutdown():
 #------------------------------------------------------------------------------
 
 def conf():
-    """
-    """
     global _Config
     return _Config
 
@@ -318,7 +312,7 @@ class DefaultsConfig(BaseConfig):
     _default = {}
 
     def setDefaultValue(self, entryPath, value):
-        self._default[entryPath] = str(value)
+        self._default[entryPath] = value
 
     def getDefaultValue(self, entryPath):
         return self._default.get(entryPath, None)
@@ -429,7 +423,8 @@ class FixedTypesConfig(NotifiableConfig):
             return {}
         elif typ == config_types.TYPE_COMBO_BOX:
             if entryPath == 'services/customer/suppliers-number':
-                return {'possible_values': ['2', '4', '7', '13', '18', '26', '64', ], }
+                from raid import eccmap
+                return {'possible_values': eccmap.SuppliersNumbers(), }
             else:
                 raise TypeError('unexpected option type for %r' % entryPath)
         return {}
@@ -451,7 +446,6 @@ class FixedTypesConfig(NotifiableConfig):
             value = self.getInt(entryPath)
         elif typ in [config_types.TYPE_FOLDER_PATH,
                      config_types.TYPE_FILE_PATH,
-                     config_types.TYPE_COMBO_BOX,
                      config_types.TYPE_PASSWORD, ]:
             value = self.getString(entryPath) or ''
             if typ in [config_types.TYPE_FOLDER_PATH, ]:
@@ -462,6 +456,11 @@ class FixedTypesConfig(NotifiableConfig):
                             value = value + u'\\'
                     elif not value.endswith(u'/'):
                         value = value + u'/'
+        elif typ == config_types.TYPE_COMBO_BOX:
+            if entryPath == 'services/customer/suppliers-number':
+                value = self.getInt(entryPath)
+            else:
+                raise TypeError('unexpected option type for %r' % entryPath)
         else:
             value = self.getData(entryPath)
         return value
@@ -486,9 +485,13 @@ class FixedTypesConfig(NotifiableConfig):
             self.setInt(entryPath, int(value))
         elif typ in [config_types.TYPE_FOLDER_PATH,
                      config_types.TYPE_FILE_PATH,
-                     config_types.TYPE_COMBO_BOX,
                      config_types.TYPE_PASSWORD, ]:
             self.setString(entryPath, value)
+        elif typ == config_types.TYPE_COMBO_BOX:
+            if entryPath == 'services/customer/suppliers-number':
+                value = self.setInt(entryPath, int(value))
+            else:
+                raise TypeError('unexpected option type for %r' % entryPath)
         else:
             self.setData(entryPath, strng.text_type(value))
         return True
