@@ -84,6 +84,9 @@ except:
 from logs import lg
 
 from system import bpio
+from system import local_fs
+
+from crypt import cipher
 
 from main import settings
 from main import events
@@ -238,8 +241,9 @@ class Initializer(automat.Automat):
         return bpio.isGUIpossible()
 
     def doUpdate(self, *args, **kwargs):
-        from main import control
-        control.request_update()
+        # from main import control
+        # control.request_update()
+        pass
 
     def doInitLocal(self, *args, **kwargs):
         """
@@ -259,7 +263,7 @@ class Initializer(automat.Automat):
         """
         if bpio.Android():
             lg.close_intercepted_log_file()
-            lg.open_intercepted_log_file('/storage/emulated/0/.bitdust/logs/android.log', mode='a')
+            lg.open_intercepted_log_file('/storage/emulated/0/Android/data/org.bitdust_io.bitdust1/files/Documents/.bitdust/logs/android.log', mode='a')
             if _Debug:
                 lg.dbg(_DebugLevel, 'log file "android.log" re-opened')
         if _Debug:
@@ -277,6 +281,12 @@ class Initializer(automat.Automat):
         #         ftp_server.init()
         #     except:
         #         lg.exc()
+        if settings.enableAPIAuthSecret():
+            current_secret = local_fs.ReadTextFile(settings.APISecretFile())
+            if not current_secret:
+                new_secret = cipher.generate_secret_text(10)
+                local_fs.WriteTextFile(settings.APISecretFile(), new_secret)
+                lg.info('generated new API auth secret text and stored in %r' % settings.APISecretFile())
         if settings.enableRESTHTTPServer():
             try:
                 from interface import api_rest_http_server
@@ -300,8 +310,8 @@ class Initializer(automat.Automat):
     def doShowGUI(self, *args, **kwargs):
         if _Debug:
             lg.out(_DebugLevel, 'initializer.doShowGUI')
-        from main import control
-        control.init()
+        # from main import control
+        # control.init()
         try:
             from system.tray_icon import USE_TRAY_ICON
         except:
