@@ -164,6 +164,15 @@ def get_info(idurl):
         return None
     return _RunningHandshakers.get(remote_idurl)
 
+
+def cancel_all():
+    global _RunningHandshakers
+    for remote_idurl in list(_RunningHandshakers.keys()):
+        h = (_RunningHandshakers.get(remote_idurl, {}) or {}).get('instance')
+        if h:
+            h.event('cancel')
+
+
 #------------------------------------------------------------------------------
 
 def on_identity_packet_outbox_status(pkt_out, status, error):
@@ -171,10 +180,11 @@ def on_identity_packet_outbox_status(pkt_out, status, error):
     remote_idurl = strng.to_bin(pkt_out.outpacket.RemoteID)
     if remote_idurl in _RunningHandshakers:
         inst = _RunningHandshakers[remote_idurl]['instance']
-        if status == 'finished':
-            inst.automat('identity-sent', pkt_out)
-        else:
-            inst.automat('outbox-failed', status=status, error=error)
+        if inst:
+            if status == 'finished':
+                inst.automat('identity-sent', pkt_out)
+            else:
+                inst.automat('outbox-failed', status=status, error=error)
 
 #------------------------------------------------------------------------------
 
