@@ -74,13 +74,6 @@ _DebugLevel = 14
 
 #------------------------------------------------------------------------------
 
-import sys
-
-try:
-    from twisted.internet import reactor  # @UnresolvedImport
-except:
-    sys.exit('Error initializing twisted.internet.reactor in online_status.py')
-
 from twisted.internet.task import LoopingCall
 from twisted.internet.defer import Deferred
 
@@ -96,6 +89,7 @@ from lib import utime
 from contacts import contactsdb
 
 from main import events
+from main import listeners
 
 from p2p import ratings
 from p2p import commands
@@ -130,7 +124,8 @@ def init():
     """
     global _OfflineCheckTask
     global _ShutdownFlag
-    lg.out(4, 'online_status.init')
+    if _Debug:
+        lg.out(_DebugLevel, 'online_status.init')
     _ShutdownFlag = False
     callback.insert_inbox_callback(1, Inbox)  # try to not overwrite top callback in the list, but stay on top
     callback.add_queue_item_status_callback(OutboxStatus)
@@ -145,7 +140,9 @@ def shutdown():
     global _OfflineCheckTask
     global _ShutdownFlag
     global _OnlineStatusDict
-    lg.out(4, 'online_status.shutdown')
+    if _Debug:
+        lg.out(_DebugLevel, 'online_status.shutdown')
+    handshaker.cancel_all()
     _OfflineCheckTask.stop()
     del _OfflineCheckTask
     _OfflineCheckTask = None
@@ -219,7 +216,7 @@ def ping(idurl, channel=None, ack_timeout=15, ping_retries=0, keep_alive=False):
     return result
 
 
-def handshake(idurl, channel=None, ack_timeout=20, ping_retries=2, keep_alive=False):
+def handshake(idurl, channel=None, ack_timeout=15, ping_retries=2, keep_alive=False):
     """
     Immediately doing handshake with remote node by fetching remote identity file and then
     sending my own Identity() to remote peer and wait for an Ack() packet.
@@ -321,8 +318,6 @@ def isCheckingNow(idurl):
 
 
 def getInstance(idurl, autocreate=True):
-    """
-    """
     if _ShutdownFlag:
         return None
     if id_url.is_empty(idurl):
@@ -450,7 +445,13 @@ def remove_online_status_listener_callback(idurl, callback_method=None, callback
     online_status_instance = getInstance(idurl)
     if not online_status_instance:
         return False
-    return online_status_instance.removeStateChangedCallback(cb=callback_method, callback_id=callback_id) > 0    
+    return online_status_instance.removeStateChangedCallback(cb=callback_method, callback_id=callback_id) > 0
+
+#------------------------------------------------------------------------------
+
+def populate_online_statuses():
+    for online_s in online_statuses().values():
+        listeners.push_snapshot('online_status', snap_id=online_s.idurl.to_text(), data=online_s.to_json())
 
 #------------------------------------------------------------------------------
 
@@ -586,7 +587,6 @@ class OnlineStatus(automat.Automat):
         Builds `online_status()` state machine.
         """
         self.idurl = idurl
-        # self.glob_id = global_id.idurl2glob(self.idurl)
         self.latest_inbox_time = None
         self.latest_check_time = None
         self.keep_alive = False
@@ -599,7 +599,18 @@ class OnlineStatus(automat.Automat):
             **kwargs
         )
         if _Debug:
-            lg.out(_DebugLevel + 2, 'online_status.ContactStatus %s %s %s' % (name, state, idurl))
+            lg.out(_DebugLevel, 'online_status.ContactStatus %s %s %s' % (name, state, idurl))
+
+    def to_json(self):
+        j = super().to_json()
+        glob_id = global_id.ParseIDURL(self.idurl)
+        j.update({
+            'idurl': self.idurl.to_text(),
+            'global_id': glob_id['customer'],
+            'idhost': glob_id['idhost'],
+            'username': glob_id['user'],
+        })
+        return j
 
     def init(self):
         """
@@ -612,7 +623,7 @@ class OnlineStatus(automat.Automat):
         Method to catch the moment when `online_status()` state were changed.
         """
         if _Debug:
-            lg.out(_DebugLevel - 2, '%s : [%s]->[%s]' % (self.name, oldstate, newstate))
+            lg.out(_DebugLevel, '%s : [%s]->[%s]' % (self.name, oldstate, newstate))
         if newstate == 'CONNECTED':
             lg.info('remote node connected : %s' % self.idurl)
             events.send('node-connected', data=dict(
@@ -621,6 +632,7 @@ class OnlineStatus(automat.Automat):
                 old_state=oldstate,
                 new_state=newstate,
             ))
+            listeners.push_snapshot('online_status', snap_id=self.idurl.to_text(), data=self.to_json())
         if newstate == 'OFFLINE' and oldstate != 'AT_STARTUP':
             lg.info('remote node disconnected : %s' % self.idurl)
             events.send('node-disconnected', data=dict(
@@ -629,6 +641,9 @@ class OnlineStatus(automat.Automat):
                 old_state=oldstate,
                 new_state=newstate,
             ))
+            listeners.push_snapshot('online_status', snap_id=self.idurl.to_text(), data=self.to_json())
+        if newstate == 'PING?' and oldstate != 'AT_STARTUP':
+            listeners.push_snapshot('online_status', snap_id=self.idurl.to_text(), data=self.to_json())
 
     def A(self, event, *args, **kwargs):
         """
@@ -724,7 +739,7 @@ class OnlineStatus(automat.Automat):
         Action method.
         """
         channel = kwargs.get('channel', None)
-        ack_timeout = kwargs.get('ack_timeout', 30)
+        ack_timeout = kwargs.get('ack_timeout', 15)
         ping_retries = kwargs.get('ping_retries', 2)
         original_idurl = kwargs.get('original_idurl', self.idurl.to_bin())
         d = None
@@ -812,20 +827,11 @@ class OnlineStatus(automat.Automat):
             err_msg = repr(err)
         if _Debug:
             lg.args(_DebugLevel, idurl=self.idurl, err=err_msg, keep_alive=self.keep_alive, handshake_callbacks=len(self.handshake_callbacks))
-        # running_handshake = handshaker.get_info(self.idurl.to_original())
-        # if not running_handshake:
-        #     running_handshake = handshaker.get_info(self.idurl.to_bin())
         for cb in self.handshake_callbacks:
             if isinstance(cb, Deferred):
                 if not cb.called:
-                    # if running_handshake:
-                    #     running_handshake['results'].append(cb)
-                    # else:
                     cb.errback(err)
             else:
-                # if running_handshake:
-                #     running_handshake['results'].append(lambda ret: cb(ret))
-                # else:
                 cb(err)
         self.handshake_callbacks = []
 
