@@ -58,9 +58,6 @@ from twisted.internet import reactor  # @UnresolvedImport
 
 from logs import lg
 
-from userid import my_id
-from userid import identity
-
 from contacts import contactsdb
 
 from contacts import identitycache
@@ -75,6 +72,12 @@ from crypt import signed
 from crypt import my_keys
 
 from transport import gateway
+from transport import packet_out
+from transport import callback
+
+from userid import identity
+from userid import id_url
+from userid import my_id
 
 #------------------------------------------------------------------------------
 
@@ -82,23 +85,26 @@ from transport import gateway
 def init():
     if _Debug:
         lg.out(_DebugLevel, 'p2p_service.init')
+    callback.insert_outbox_filter_callback(0, outbox)
 
 
 def shutdown():
     if _Debug:
         lg.out(_DebugLevel, 'p2p_service.shutdown')
+    callback.remove_outbox_filter_callback(outbox)
 
 #------------------------------------------------------------------------------
 
-
 def inbox(newpacket, info, status, error_message):
-    """
-    """
-    if newpacket.CreatorID != my_id.getIDURL() and newpacket.RemoteID != my_id.getIDURL():
-        # packet is NOT for us, skip
-        return False
+#     if newpacket.CreatorID != my_id.getIDURL() and newpacket.RemoteID != my_id.getIDURL():
+#         # packet is NOT for us, skip
+#         return False
 
-    if newpacket.Command == commands.Ack():
+    if newpacket.Command == commands.Identity():
+        # a response from remote node, typically handled in other places
+        Identity(newpacket, info)
+
+    elif newpacket.Command == commands.Ack():
         # a response from remote node, typically handled in other places
         Ack(newpacket, info)
 
@@ -164,6 +170,10 @@ def inbox(newpacket, info, status, error_message):
         # handled by service_keys_registry()
         Key(newpacket, info)
 
+    elif newpacket.Command == commands.AuditKey():
+        # handled by service_keys_registry()
+        AuditKey(newpacket, info)
+
     elif newpacket.Command == commands.Event():
         # handled by service_p2p_hookups()
         Event(newpacket, info)
@@ -176,13 +186,17 @@ def inbox(newpacket, info, status, error_message):
         # handled by service_customer_family()
         Contacts(newpacket, info)
 
+    else:
+        lg.warn('unexpected command received: %r from %r' % (newpacket, info, ))
+
     return False
 
 
-def outbox(outpacket):
+def outbox(outpacket, wide, callbacks, target=None, route=None, response_timeout=None, keep_alive=True):
     if _Debug:
-        lg.out(_DebugLevel, "p2p_service.outbox [%s] to %s" % (outpacket.Command, nameurl.GetName(outpacket.RemoteID)))
-    return True
+        lg.out(_DebugLevel, "p2p_service.outbox [%s:%s] to %s with route %r" % (
+            outpacket.Command, outpacket.PacketID, nameurl.GetName(outpacket.RemoteID), route, ))
+    return None
 
 #------------------------------------------------------------------------------
 
@@ -231,11 +245,10 @@ def SendAckNoRequest(remoteID, packetid, response='', wide=False, callbacks={}):
 
 
 def Fail(newpacket):
-    if newpacket.Payload:
-        if _Debug:
+    if _Debug:
+        if newpacket.Payload:
             lg.warn('%r received from [%s] in %r' % (newpacket.Payload, newpacket.CreatorID, newpacket))
-    else:
-        if _Debug:
+        else:
             lg.out(_DebugLevel, "p2p_service.Fail from %s|%s packetID=%s : %s" % (
                 newpacket.CreatorID, newpacket.OwnerID, newpacket.PacketID, newpacket.Payload))
 
@@ -274,61 +287,79 @@ def SendFailNoRequest(remoteID, packetID, response=''):
 
 #------------------------------------------------------------------------------
 
-def Identity(newpacket, send_ack=True):
-    """
-    Normal node or Identity server is sending us a new copy of an identity for a contact of ours.
-    Checks that identity is signed correctly.
-    Sending requests to cache all sources (other identity servers) holding that identity.
-    """
-    # TODO:  move to service_gateway
-    newxml = newpacket.Payload
-    newidentity = identity.identity(xmlsrc=newxml)
-    # SECURITY
-    # check that identity is signed correctly
-    # old public key matches new one
-    # this is done in `UpdateAfterChecking()`
-    idurl = newidentity.getIDURL()
-    if not identitycache.HasKey(idurl):
-        lg.info('received new identity %s rev %r' % (idurl, newidentity.getRevisionValue()))
-    if not identitycache.UpdateAfterChecking(idurl, newxml):
-        lg.warn("ERROR has non-Valid identity")
-        return False
-    # Now that we have ID we can check packet
-    if not newpacket.Valid():
-        # If not valid do nothing
-        lg.warn("not Valid packet from %s" % idurl)
-        return False
-    # TODO: after receiving full list of identity sources we can call ALL OF THEM or those which are not cached yet.
-    # this way we can be sure that even if first source (server holding your public key) is not responding
-    # other sources still can give you required user info: public key, contacts, etc..
-    # TODO: we can also consolidate few "idurl" sources for every public key - basically identify user by public key
-    # something like:
-    # for source in identitycache.FromCache(idurl).getSources(as_originals=True):
-    #     if source not in identitycache.FromCache(idurl):
-    #         d = identitycache.immediatelyCaching(source)
-    #         d.addCallback(lambda xml_src: identitycache.UpdateAfterChecking(idurl, xml_src))
-    #         d.addErrback(lambda err: lg.warn('caching filed: %s' % err))
-    if not send_ack:
-        if _Debug:
-            lg.out(_DebugLevel, "p2p_service.Identity %s  idurl=%s  remoteID=%r  skip sending Ack()" % (
-                newpacket.PacketID, idurl, newpacket.RemoteID))
-        return True
-    if newpacket.OwnerID == idurl:
-        if _Debug:
-            lg.out(_DebugLevel, "p2p_service.Identity %s  idurl=%s  remoteID=%r  sending wide Ack()" % (
-                newpacket.PacketID, idurl, newpacket.RemoteID))
-    else:
-        if _Debug:
-            lg.out(_DebugLevel, "p2p_service.Identity %s  idurl=%s  remoteID=%r  but packet ownerID=%s   sending wide Ack()" % (
-                newpacket.PacketID, idurl, newpacket.RemoteID, newpacket.OwnerID, ))
-    # wide=True : a small trick to respond to all his contacts
-    reactor.callLater(0, SendAck, newpacket, wide=True)  # @UndefinedVariable
-    return True
+def Identity(newpacket, info):
+    if _Debug:
+        lg.out(_DebugLevel, "p2p_service.Identity %s from [%s] at %s://%s with %d bytes payload" % (
+            newpacket.PacketID, nameurl.GetName(newpacket.CreatorID),
+            info.proto, info.host, len(newpacket.Payload)))
+
+
+#     """
+#     Normal node or Identity server is sending us a new copy of an identity for a contact of ours.
+#     Checks that identity is signed correctly.
+#     Sending requests to cache all sources (other identity servers) holding that identity.
+#     """
+#     # TODO:  move to service_gateway
+#     newxml = newpacket.Payload
+#     newidentity = identity.identity(xmlsrc=newxml)
+#     # SECURITY
+#     # check that identity is signed correctly
+#     # old public key matches new one
+#     # this is done in `UpdateAfterChecking()`
+#     idurl = newidentity.getIDURL()
+#     if not identitycache.HasKey(idurl):
+#         lg.info('received new identity %s rev %r' % (idurl, newidentity.getRevisionValue()))
+#     if not identitycache.UpdateAfterChecking(idurl, newxml):
+#         lg.warn("ERROR has non-Valid identity")
+#         return False
+#     if my_id.isLocalIdentityReady():
+#         if newidentity.getPublicKey() == my_id.getLocalIdentity().getPublicKey():
+#             if newidentity.getRevisionValue() > my_id.getLocalIdentity().getRevisionValue():
+#                 lg.warn('received my own identity from another user, but with higher revision number')
+#                 reactor.callLater(0, my_id.rebuildLocalIdentity, new_revision=newidentity.getRevisionValue() + 1)  # @UndefinedVariable
+#                 return False
+#     latest_identity = id_url.get_latest_ident(newidentity.getPublicKey())
+#     if latest_identity:
+#         if latest_identity.getRevisionValue() > newidentity.getRevisionValue():
+#             # check if received identity is the most recent revision number we every saw for that remote user
+#             # in case we saw same identity with higher revision number need to reply with Fail packet and notify user
+#             # this may happen after identity restore - the user starts counting revision number from 0
+#             # but other nodes already store previous copies, user just need to jump to the most recent revision number
+#             lg.warn('received new identity with out-dated revision number from %r' % idurl)
+#             ident_packet = signed.Packet(
+#                 Command=commands.Identity(),
+#                 OwnerID=latest_identity.getIDURL(),
+#                 CreatorID=latest_identity.getIDURL(),
+#                 PacketID='identity:%s' % packetid.UniqueID(),
+#                 Payload=latest_identity.serialize(),
+#                 RemoteID=idurl,
+#             )
+#             reactor.callLater(0, packet_out.create, outpacket=ident_packet, wide=True, callbacks={}, keep_alive=False)  # @UndefinedVariable
+#             return False
+#     # Now that we have ID we can check the packet
+#     if not newpacket.Valid():
+#         # If not valid do nothing
+#         lg.warn("not Valid packet from %s" % idurl)
+#         return False
+#     if not send_ack:
+#         if _Debug:
+#             lg.out(_DebugLevel, "p2p_service.Identity %s  idurl=%s  remoteID=%r  skip sending Ack()" % (
+#                 newpacket.PacketID, idurl, newpacket.RemoteID))
+#         return True
+#     if newpacket.OwnerID == idurl:
+#         if _Debug:
+#             lg.out(_DebugLevel, "p2p_service.Identity %s  idurl=%s  remoteID=%r  sending wide Ack()" % (
+#                 newpacket.PacketID, idurl, newpacket.RemoteID))
+#     else:
+#         if _Debug:
+#             lg.out(_DebugLevel, "p2p_service.Identity %s  idurl=%s  remoteID=%r  but packet ownerID=%s   sending wide Ack()" % (
+#                 newpacket.PacketID, idurl, newpacket.RemoteID, newpacket.OwnerID, ))
+#     # wide=True : a small trick to respond to all known contacts of the remote user
+#     reactor.callLater(0, SendAck, newpacket, wide=True)  # @UndefinedVariable
+#     return True
 
 
 def SendIdentity(remote_idurl, wide=False, timeout=10, callbacks={}):
-    """
-    """
     packet_id = 'identity:%s' % packetid.UniqueID()
     if _Debug:
         lg.out(_DebugLevel, "p2p_service.SendIdentity to %s wide=%s packet_id=%r" % (
@@ -348,14 +379,12 @@ def SendIdentity(remote_idurl, wide=False, timeout=10, callbacks={}):
 
 
 def RequestService(request, info):
-    """
-    """
-    try:
-        service_info = serialization.BytesToDict(request.Payload)
-    except:
-        lg.exc()
-        service_info = {}
     if _Debug:
+        try:
+            service_info = serialization.BytesToDict(request.Payload)
+        except:
+            lg.exc()
+            service_info = {}
         lg.out(_DebugLevel, 'p2p_service.RequestService %s with "%s" in %d bytes' % (
             request.PacketID, service_info.get('name', 'unknown service name'), len(request.Payload)))
         lg.out(_DebugLevel, '  from remoteID=%s  ownerID=%s  creatorID=%s' % (
@@ -386,12 +415,12 @@ def SendRequestService(remote_idurl, service_name, json_payload={}, wide=False,
 
 
 def CancelService(request, info):
-    try:
-        service_info = serialization.BytesToDict(request.Payload)
-    except:
-        lg.exc()
-        service_info = {}
     if _Debug:
+        try:
+            service_info = serialization.BytesToDict(request.Payload)
+        except:
+            lg.exc()
+            service_info = {}
         lg.out(_DebugLevel, 'p2p_service.CancelService %s with "%s" in %d bytes' % (
             request.PacketID, service_info.get('name', 'unknown service name'), len(request.Payload), ))
         lg.out(_DebugLevel, '  from remoteID=%s  ownerID=%s  creatorID=%s' % (
@@ -529,8 +558,6 @@ def Data(request):
 
 
 def SendData(raw_data, ownerID, creatorID, remoteID, packetID, callbacks={}):
-    """
-    """
     newpacket = signed.Packet(
         Command=commands.Data(),
         OwnerID=ownerID,
@@ -561,8 +588,6 @@ def Retrieve(request):
 
 
 def SendRetreive(ownerID, creatorID, packetID, remoteID, payload='', response_timeout=None, callbacks={}):
-    """
-    """
     newpacket = signed.Packet(
         Command=commands.Retrieve(),
         OwnerID=ownerID,
@@ -803,8 +828,6 @@ def SendRetrieveCoin(remote_idurl, query, wide=False, callbacks={}):
 #------------------------------------------------------------------------------
 
 def Key(request, info):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'p2p_service.Key %d bytes in [%s]' % (len(request.Payload), request.PacketID))
         lg.out(_DebugLevel, '  from senderID=%s to remoteID=%s  ownerID=%s  creatorID=%s  ' % (
@@ -830,8 +853,6 @@ def SendKey(remote_idurl, encrypted_key_data, packet_id=None, wide=False, callba
 
 
 def AuditKey(request, info):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'p2p_service.AuditKey %d bytes in [%s]' % (len(request.Payload), request.PacketID))
         lg.out(_DebugLevel, '  from remoteID=%s  ownerID=%s  creatorID=%s  sender_idurl=%s' % (
@@ -858,8 +879,6 @@ def SendAuditKey(remote_idurl, encrypted_payload, packet_id=None, timeout=10, wi
 #------------------------------------------------------------------------------
 
 def Event(request, info):
-    """
-    """
     if _Debug:
         try:
             e_json = serialization.BytesToDict(request.Payload, keys_to_text=True)
@@ -910,8 +929,6 @@ def SendEvent(remote_idurl, event_id, payload=None,
 #------------------------------------------------------------------------------
 
 def Message(request, info):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'p2p_service.Message %d bytes in [%s]' % (len(request.Payload), request.PacketID))
         lg.out(_DebugLevel, '  from remoteID=%s  ownerID=%s  creatorID=%s' % (
@@ -919,8 +936,6 @@ def Message(request, info):
 
 
 def SendMessage(remote_idurl, packet_id=None, payload=None, wide=True, callbacks={}, response_timeout=None):
-    """
-    """
     if packet_id is None:
         packet_id = packetid.UniqueID()
     if _Debug:
@@ -939,8 +954,6 @@ def SendMessage(remote_idurl, packet_id=None, payload=None, wide=True, callbacks
 #------------------------------------------------------------------------------
 
 def Contacts(request, info):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'p2p_service.Contacts %d bytes in [%s]' % (len(request.Payload), request.PacketID))
         lg.out(_DebugLevel, '  from remoteID=%s  ownerID=%s  creatorID=%s' % (
@@ -948,8 +961,6 @@ def Contacts(request, info):
 
 
 def SendContacts(remote_idurl, json_payload={}, wide=False, callbacks={}):
-    """
-    """
     MyID = my_id.getIDURL()
     if _Debug:
         lg.out(_DebugLevel, "p2p_service.SendContacts to %s" % nameurl.GetName(remote_idurl))
