@@ -411,8 +411,6 @@ def HandleTimeOut(pkt_out):
 
 
 def OnFileSent(pkt_out, item, status, size, error_message):
-    """
-    """
     return False
 
 
@@ -457,14 +455,14 @@ def SendToIDs(idlist, wide=False, ack_handler=None, timeout_handler=None, respon
     Same, but send to many IDs and also check previous packets to not re-send.
     """
     global _PropagateCounter
-    if _Debug:
-        lg.out(_DebugLevel, "propagate.SendToIDs to %d users, wide=%s" % (len(idlist), wide))
     if ack_handler is None:
         ack_handler = HandleAck
     if timeout_handler is None:
         timeout_handler = HandleTimeOut
     LocalIdentity = my_id.getLocalIdentity()
     Payload = strng.to_bin(LocalIdentity.serialize())
+    if _Debug:
+        lg.out(_DebugLevel, "propagate.SendToIDs to %d users, rev=%r wide=%s" % (len(idlist), LocalIdentity.getRevisionValue(), wide))
     alreadysent = set()
     totalsent = 0
     inqueue = {}
@@ -525,7 +523,7 @@ def SendToIDs(idlist, wide=False, ack_handler=None, timeout_handler=None, respon
 
 #------------------------------------------------------------------------------
 
-def ping_suppliers(customer_idurl=None, timeout=30):
+def ping_suppliers(customer_idurl=None, timeout=20):
     from p2p import online_status
     l = []
     for supplier_idurl in contactsdb.suppliers(customer_idurl=customer_idurl):
@@ -534,7 +532,7 @@ def ping_suppliers(customer_idurl=None, timeout=30):
     return DeferredList(l, consumeErrors=True)
 
 
-def ping_customers(timeout=30):
+def ping_customers(timeout=20):
     from p2p import online_status
     l = []
     for customer_idurl in contactsdb.customers():
