@@ -69,6 +69,7 @@ from system import bpio
 
 from main import config
 from main import events
+from main import listeners
 
 #------------------------------------------------------------------------------
 
@@ -120,7 +121,7 @@ def is_started(name):
     svc = services().get(name, None)
     if svc is None:
         return False
-    return svc.state != 'ON' and svc.state != 'OFF' and svc.state != 'NOT_INSTALLED' and svc.state != 'DEPENDS_OFF'
+    return svc.state == 'ON' or svc.state == 'STOPPING' or svc.state == 'STARTING' or svc.state == 'INFLUENCE'
 
 
 def is_enabled(name):
@@ -225,8 +226,6 @@ def cancel(service_name, service_cancel_payload, request, info):
 
 
 def init():
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'driver.init')
     available_services_dir = os.path.join(bpio.getExecutableDir(), 'services')
@@ -270,8 +269,6 @@ def init():
 
 
 def shutdown():
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'driver.shutdown')
     config.conf().removeConfigNotifier('services/')
@@ -286,8 +283,6 @@ def shutdown():
 
 
 def build_order():
-    """
-    """
     global _BootUpOrder
     order = list(enabled_services())
     progress = True
@@ -326,8 +321,6 @@ def build_order():
 
 
 def start(services_list=[]):
-    """
-    """
     global _StartingDeferred
     global _StopingDeferred
     if _Debug:
@@ -365,8 +358,6 @@ def start(services_list=[]):
 
 
 def stop(services_list=[]):
-    """
-    """
     global _StopingDeferred
     global _StartingDeferred
     if _Debug:
@@ -428,8 +419,6 @@ def info(service_name):
 #------------------------------------------------------------------------------
 
 def restart(service_name, wait_timeout=None):
-    """
-    """
     global _StopingDeferred
     global _StartingDeferred
     restart_result = Deferred()
@@ -663,10 +652,34 @@ def get_attached_dht_layers(services_list=[]):
 
 #------------------------------------------------------------------------------
 
+def populate_services():
+    services_list = reversed(boot_up_order())
+    for name in services_list:
+        svc = services().get(name, None)
+        if not svc:
+            continue
+        svc_data = svc.to_json()
+        svc_data['event'] = None
+        listeners.push_snapshot('service', snap_id=name, data=svc_data)
+
+#------------------------------------------------------------------------------
+
+def do_finish_starting():
+    global _StartingDeferred
+    if _Debug:
+        lg.args(_DebugLevel, starting=bool(_StartingDeferred))
+    _StartingDeferred = None
+
+
+def do_finish_stoping():
+    global _StopingDeferred
+    if _Debug:
+        lg.args(_DebugLevel, stoping=bool(_StopingDeferred))
+    _StopingDeferred = None
+
+#------------------------------------------------------------------------------
 
 def on_service_callback(result, service_name):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'driver.on_service_callback %s : [%s]' % (service_name, result))
     svc = services().get(service_name, None)
@@ -694,6 +707,8 @@ def on_service_callback(result, service_name):
                     continue
                 if relative_service.state == 'ON':
                     continue
+                if _Debug:
+                    lg.out(_DebugLevel, '    making attempt to start relative service %r' % relative_service)
                 relative_service.automat('start')
     elif result == 'stopped':
         if _Debug:
@@ -712,23 +727,12 @@ def on_service_callback(result, service_name):
             if not depend_service:
                 raise ServiceNotFound(depend_name)
             depend_service.automat('depend-service-stopped')
+    svc_data = svc.to_json()
+    svc_data['event'] = result
+    listeners.push_snapshot('service', snap_id=service_name, data=svc_data)
     return result
 
 
-def do_finish_starting():
-    global _StartingDeferred
-    if _Debug:
-        lg.args(_DebugLevel, starting=bool(_StartingDeferred))
-    _StartingDeferred = None
-
-
-def do_finish_stoping():
-    global _StopingDeferred
-    if _Debug:
-        lg.args(_DebugLevel, stoping=bool(_StopingDeferred))
-    _StopingDeferred = None
-
-
 def on_started_all_services(results):
     if _Debug:
         lg.out(_DebugLevel, 'driver.on_started_all_services results=%d' % len(results))
