@@ -72,11 +72,9 @@ from lib import misc
 from lib import packetid
 from lib import nameurl
 from lib import strng
-from lib import jsn
 
 from main import settings
 from main import events
-from main import control
 
 from raid import eccmap
 
@@ -90,7 +88,6 @@ from userid import id_url
 from services import driver
 
 from contacts import contactsdb
-from contacts import identitycache
 
 from p2p import p2p_service
 
@@ -109,7 +106,6 @@ MAXIMUM_JOBS_STARTED = 1  # let's do only one backup at once for now
 _Jobs = {}   # here are already started backups ( by backupID )
 _Tasks = []  # here are tasks to start backups in the future ( pathID )
 _LastTaskNumber = 0
-_RevisionNumber = 0
 _LoadingFlag = False
 _TaskStartedCallbacks = {}
 _TaskFinishedCallbacks = {}
@@ -132,27 +128,6 @@ def tasks():
     global _Tasks
     return _Tasks
 
-
-def revision():
-    """
-    Mutator method to access current software revision number.
-    """
-    global _RevisionNumber
-    return _RevisionNumber
-
-
-def commit(new_revision_number=None):
-    """
-    Need to be called after any changes in the index database.
-
-    This increase revision number by 1 or set ``new_revision_number``.
-    """
-    global _RevisionNumber
-    if new_revision_number:
-        _RevisionNumber = new_revision_number
-    else:
-        _RevisionNumber += 1
-
 #------------------------------------------------------------------------------
 
 
@@ -164,7 +139,6 @@ def init():
     """
     if _Debug:
         lg.out(_DebugLevel, 'backup_control.init')
-    Load()
 
 
 def shutdown():
@@ -176,138 +150,22 @@ def shutdown():
 
 #------------------------------------------------------------------------------
 
-
-def WriteIndex(filepath=None, encoding='utf-8'):
+def Save(customer_idurl=None, key_alias='master', increase_revision=True):
     """
-    Write index data base to the local file .bitdust/metadata/index.
+    Save index data base to local file and notify "index_synchronizer()" state machine.
     """
     global _LoadingFlag
     if _LoadingFlag:
-        return
-    if filepath is None:
-        filepath = settings.BackupIndexFilePath()
-    json_data = {}
-    # json_data = backup_fs.Serialize(to_json=True, encoding=encoding)
-    for customer_idurl in backup_fs.known_customers():
-        # customer_id = global_id.UrlToGlobalID(customer_idurl)
-        customer_id = customer_idurl.to_id()
-        json_data[customer_id] = backup_fs.Serialize(
-            iterID=backup_fs.fsID(customer_idurl),
-            to_json=True,
-            encoding=encoding,
+        return False
+    if customer_idurl is None:
+        customer_idurl = my_id.getIDURL()
+    customer_idurl = id_url.field(customer_idurl)
+    if increase_revision:
+        backup_fs.commit(
+            customer_idurl=customer_idurl,
+            key_alias=key_alias,
         )
-    src = '%d\n' % revision()
-    src += jsn.dumps(
-        json_data,
-        indent=1,
-        separators=(',', ':'),
-        encoding=encoding,
-    )
-    if _Debug:
-        lg.args(_DebugLevel, size=len(src), filepath=filepath)
-    return bpio.WriteTextFile(filepath, src)
-
-
-def ReadIndex(text_data, encoding='utf-8'):
-    """
-    Read index data base, ``input`` is a ``StringIO.StringIO`` object which
-    keeps the data.
-
-    This is a simple text format, see ``p2p.backup_fs.Serialize()``
-    method. The first line keeps revision number.
-    """
-    global _LoadingFlag
-    if _LoadingFlag:
-        return False
-    _LoadingFlag = True
-    backup_fs.Clear()
-    count = 0
-    try:
-        json_data = jsn.loads(
-            text_data,
-            encoding=encoding,
-        )
-    except:
-        lg.exc()
-        json_data = text_data
-    if _Debug:
-        lg.args(_DebugLevel, json_data=json_data)
-    for customer_id in json_data.keys():
-        if customer_id == 'items':
-            try:
-                count = backup_fs.Unserialize(json_data, from_json=True, decoding=encoding)
-            except:
-                lg.exc()
-                return False
-        else:
-            customer_idurl = global_id.GlobalUserToIDURL(customer_id)
-            if not id_url.is_cached(customer_idurl):
-                lg.warn('identity %r is not yet cached, skip reading related catalog items' % customer_idurl)
-                identitycache.immediatelyCaching(customer_idurl, try_other_sources=False, ignore_errors=True)
-                continue
-            try:
-                count = backup_fs.Unserialize(
-                    json_data[customer_id],
-                    iter=backup_fs.fs(customer_idurl),
-                    iterID=backup_fs.fsID(customer_idurl),
-                    from_json=True,
-                    decoding=encoding,
-                )
-            except:
-                lg.exc()
-                return False
-    if _Debug:
-        lg.out(_DebugLevel, 'backup_control.ReadIndex %d items loaded' % count)
-    # local_site.update_backup_fs(backup_fs.ListAllBackupIDsSQL())
-    # commit(new_revision)
-    _LoadingFlag = False
-    return True
-
-
-def Load(filepath=None):
-    """
-    This load the data from local file and call ``ReadIndex()`` method.
-    """
-    global _LoadingFlag
-    if _LoadingFlag:
-        return False
-    if filepath is None:
-        filepath = settings.BackupIndexFilePath()
-    if not os.path.isfile(filepath):
-        lg.warn('file %s not exist' % filepath)
-        WriteIndex(filepath)
-    src = bpio.ReadTextFile(filepath)
-    if not src:
-        lg.out(2, 'backup_control.Load ERROR reading file %s' % filepath)
-        return False
-    inpt = StringIO(src)
-    try:
-        known_revision = int(inpt.readline().rstrip('\n'))
-    except:
-        lg.exc()
-        return False
-    raw_data = inpt.read()
-    inpt.close()
-    ret = ReadIndex(raw_data)
-    if ret:
-        commit(known_revision)
-        backup_fs.Scan()
-        backup_fs.Calculate()
-    else:
-        lg.warn('catalog index reading failed')
-    return ret
-
-
-def Save(filepath=None):
-    """
-    Save index data base to local file ( call ``WriteIndex()`` ) and notify
-    "index_synchronizer()" state machine.
-    """
-    global _LoadingFlag
-    if _LoadingFlag:
-        return False
-    commit()
-    WriteIndex(filepath)
+    backup_fs.SaveIndex(customer_idurl, key_alias)
     if driver.is_on('service_backup_db'):
         # TODO: switch to event
         from storage import index_synchronizer
@@ -320,7 +178,12 @@ def on_files_received(newpacket, info):
     if not list_files_global_id['idurl']:
         lg.warn('invalid PacketID: %s' % newpacket.PacketID)
         return False
-    if list_files_global_id['idurl'] != my_id.getIDURL():
+    if list_files_global_id['key_alias'] != 'master':
+        # ignore Files() if this is a shared data
+        if _Debug:
+            lg.dbg(_DebugLevel, 'ignore incoming %r, this is not a private data' % newpacket)
+        return False
+    if not id_url.is_the_same(list_files_global_id['idurl'], my_id.getIDURL()):
         # ignore Files() if this is another customer
         if _Debug:
             lg.dbg(_DebugLevel, 'ignore incoming %r which is owned by another customer' % newpacket)
@@ -401,53 +264,54 @@ def IncomingSupplierListFiles(newpacket, list_files_global_id):
     return True
 
 
-def IncomingSupplierBackupIndex(newpacket):
+def IncomingSupplierBackupIndex(newpacket, key_id=None):
     """
     Called by ``p2p.p2p_service`` when a remote copy of our local index data
     base ( in the "Data" packet ) is received from one of our suppliers.
 
     The index is also stored on suppliers to be able to restore it.
     """
-    b = encrypted.Unserialize(newpacket.Payload)
-    if b is None:
-        lg.out(2, 'backup_control.IncomingSupplierBackupIndex ERROR reading data from %s' % newpacket.RemoteID)
+    block = encrypted.Unserialize(newpacket.Payload, decrypt_key=key_id)
+    if not block:
+        lg.err('failed reading data from %s' % newpacket.RemoteID)
         return None
     try:
-        session_key = key.DecryptLocalPrivateKey(b.EncryptedSessionKey)
-        padded_data = key.DecryptWithSessionKey(session_key, b.EncryptedData, session_key_type=b.SessionKeyType)
-        inpt = StringIO(strng.to_text(padded_data[:int(b.Length)]))
+        data = block.Data()
+        inpt = StringIO(strng.to_text(data))
         supplier_revision = inpt.readline().rstrip('\n')
         if supplier_revision:
             supplier_revision = int(supplier_revision)
         else:
-            supplier_revision = -1
+            supplier_revision = None
+        text_data = inpt.read()
     except:
-        lg.out(2, 'backup_control.IncomingSupplierBackupIndex ERROR reading data from %s' % newpacket.RemoteID)
         lg.exc()
         try:
             inpt.close()
         except:
             pass
         return None
-    if revision() > supplier_revision:
-        inpt.close()
-        if _Debug:
-            lg.out(_DebugLevel, 'backup_control.IncomingSupplierBackupIndex SKIP, supplier %s revision=%d, local revision=%d' % (
-                newpacket.RemoteID, supplier_revision, revision(), ))
-        return supplier_revision
-    text_data = inpt.read()
     inpt.close()
-    if ReadIndex(text_data):
-        commit(supplier_revision)
-        backup_fs.Scan()
-        backup_fs.Calculate()
-        WriteIndex()
-        control.request_update()
-        if _Debug:
-            lg.out(_DebugLevel, 'backup_control.IncomingSupplierBackupIndex updated to revision %d from %s' % (
-                revision(), newpacket.RemoteID))
-    else:
-        lg.warn('failed to read catalog index from supplier')
+#     if supplier_revision >= 0 and backup_fs.revision() > supplier_revision:
+#         if _Debug:
+#             lg.out(_DebugLevel, 'backup_control.IncomingSupplierBackupIndex SKIP, supplier %s revision=%d, local revision=%d' % (
+#                 newpacket.RemoteID, supplier_revision, backup_fs.revision(), ))
+#         return supplier_revision
+    if _Debug:
+        lg.args(_DebugLevel, k=key_id, p=newpacket.PacketID, sz=len(text_data), inp=len(newpacket.Payload))
+    count, updated_customers_keys = backup_fs.ReadIndex(text_data, new_revision=supplier_revision)
+    if updated_customers_keys:
+        # backup_fs.commit(supplier_revision)
+        # backup_fs.Scan()
+        # backup_fs.Calculate()
+        for customer_idurl, key_alias in updated_customers_keys:
+            backup_fs.SaveIndex(customer_idurl, key_alias)
+            # control.request_update()
+            if _Debug:
+                lg.out(_DebugLevel, 'backup_control.IncomingSupplierBackupIndex updated to revision %d for %s of %s from %s' % (
+                    backup_fs.revision(customer_idurl, key_alias), customer_idurl, key_alias, newpacket.RemoteID))
+    # else:
+    #     lg.warn('failed to read catalog index from supplier')
     return supplier_revision
 
 #------------------------------------------------------------------------------
@@ -458,7 +322,7 @@ def DeleteAllBackups():
     Remove all backup IDs from index data base, see ``DeleteBackup()`` method.
     """
     # prepare a list of all known backup IDs
-    all_ids = set(backup_fs.ListAllBackupIDs())
+    all_ids = set(backup_fs.ListAllBackupIDs(customer_idurl=my_id.getIDURL()))
     all_ids.update(backup_matrix.GetBackupIDs(remote=True, local=True))
     if _Debug:
         lg.out(_DebugLevel, 'backup_control.DeleteAllBackups %d ID\'s to kill' % len(all_ids))
@@ -472,7 +336,7 @@ def DeleteAllBackups():
     # save the index
     Save()
     # refresh the GUI
-    control.request_update()
+    # control.request_update()
 
 
 def DeleteBackup(backupID, removeLocalFilesToo=True, saveDB=True, calculate=True):
@@ -494,11 +358,13 @@ def DeleteBackup(backupID, removeLocalFilesToo=True, saveDB=True, calculate=True
     # if the user deletes a backup, make sure we remove any work we're doing on it
     # abort backup if it just started and is running at the moment
     if AbortRunningBackup(backupID):
-        lg.out(8, 'backup_control.DeleteBackup %s is in process, stopping' % backupID)
+        if _Debug:
+            lg.out(_DebugLevel, 'backup_control.DeleteBackup %s is in process, stopping' % backupID)
         return True
     from stream import io_throttle
     from storage import backup_rebuilder
-    lg.out(8, 'backup_control.DeleteBackup ' + backupID)
+    if _Debug:
+        lg.out(_DebugLevel, 'backup_control.DeleteBackup ' + backupID)
     # if we requested for files for this backup - we do not need it anymore
     io_throttle.DeleteBackupRequests(backupID)
     io_throttle.DeleteBackupSendings(backupID)
@@ -524,7 +390,7 @@ def DeleteBackup(backupID, removeLocalFilesToo=True, saveDB=True, calculate=True
     # in some cases we want to save the DB later
     if saveDB:
         Save()
-        control.request_update([('backupID', backupID), ])
+        # control.request_update([('backupID', backupID), ])
     return True
 
 
@@ -538,16 +404,19 @@ def DeletePathBackups(pathID, removeLocalFilesToo=True, saveDB=True, calculate=T
     pathID = global_id.CanonicalID(pathID)
     # get the working item
     customer, remotePath = packetid.SplitPacketID(pathID)
+    key_alias = packetid.KeyAlias(customer)
     customer_idurl = global_id.GlobalUserToIDURL(customer)
-    item = backup_fs.GetByID(remotePath, iterID=backup_fs.fsID(customer_idurl))
+    item = backup_fs.GetByID(remotePath, iterID=backup_fs.fsID(customer_idurl, key_alias))
     if item is None:
         return False
-    lg.out(8, 'backup_control.DeletePathBackups ' + pathID)
+    if _Debug:
+        lg.out(_DebugLevel, 'backup_control.DeletePathBackups ' + pathID)
     # this is a list of all known backups of this path
     versions = item.list_versions()
     for version in versions:
-        backupID = packetid.MakeBackupID(customer, remotePath, version)
-        lg.out(8, '        removing %s' % backupID)
+        backupID = packetid.MakeBackupID(customer, remotePath, version, key_alias=key_alias)
+        if _Debug:
+            lg.out(_DebugLevel, '        removing %s' % backupID)
         # abort backup if it just started and is running at the moment
         AbortRunningBackup(backupID)
         # if we requested for files for this backup - we do not need it anymore
@@ -575,7 +444,7 @@ def DeletePathBackups(pathID, removeLocalFilesToo=True, saveDB=True, calculate=T
     # save the index if needed
     if saveDB:
         Save()
-        control.request_update()
+        # control.request_update()
     return True
 
 #------------------------------------------------------------------------------
@@ -653,7 +522,7 @@ class Task():
         ))
 
     def set_path_id(self, pathID):
-        parts = global_id.ParseGlobalID(pathID)
+        parts = global_id.NormalizeGlobalID(pathID)
         self.pathID = pathID  # source path to backup
         self.customerGlobID = parts['customer']
         self.customerIDURL = parts['idurl']
@@ -685,7 +554,7 @@ class Task():
         """
         Runs a new ``Job`` from that ``Task``.
         """
-        iter_and_path = backup_fs.WalkByID(self.remotePath, iterID=backup_fs.fsID(self.customerIDURL))
+        iter_and_path = backup_fs.WalkByID(self.remotePath, iterID=backup_fs.fsID(self.customerIDURL, self.keyAlias))
         if iter_and_path is None:
             if _Debug:
                 lg.out(_DebugLevel, 'backup_control.Task.run ERROR %s not found in the index' % self.remotePath)
@@ -764,8 +633,13 @@ class Task():
             sz = os.path.getsize(self.localPath)
             jobs()[self.backupID].totalSize = sz
             itemInfo.set_size(sz)
-            backup_fs.Calculate()
-            Save()
+            backup_fs.Calculate(iterID=backup_fs.fsID(self.customerIDURL, self.keyAlias))
+            backup_fs.SaveIndex(customer_idurl=self.customerIDURL, key_alias=self.keyAlias)
+            if self.keyAlias == 'master':
+                if driver.is_on('service_backup_db'):
+                    # TODO: switch to event
+                    from storage import index_synchronizer
+                    index_synchronizer.A('push')
         jobs()[self.backupID].automat('start')
         reactor.callLater(0, FireTaskStartedCallbacks, self.pathID, dataID)  # @UndefinedVariable
         if _Debug:
@@ -842,8 +716,6 @@ def HasTask(pathID):
 
 
 def GetPendingTask(pathID):
-    """
-    """
     pathID = global_id.CanonicalID(pathID)
     for t in tasks():
         if t.pathID == pathID:
@@ -852,14 +724,10 @@ def GetPendingTask(pathID):
 
 
 def ListPendingTasks():
-    """
-    """
     return tasks()
 
 
 def AbortPendingTask(pathID):
-    """
-    """
     pathID = global_id.CanonicalID(pathID)
     for t in tasks():
         if t.pathID == pathID:
@@ -878,13 +746,14 @@ def OnFoundFolderSize(pth, sz, arg):
         pathID, version = arg
         customerGlobID, pathID = packetid.SplitPacketID(pathID)
         customerIDURL = global_id.GlobalUserToIDURL(customerGlobID)
-        item = backup_fs.GetByID(pathID, iterID=backup_fs.fsID(customerIDURL))
+        keyAlias = packetid.KeyAlias(customerGlobID)
+        item = backup_fs.GetByID(pathID, iterID=backup_fs.fsID(customerIDURL, keyAlias))
         if item:
             item.set_size(sz)
             backup_fs.Calculate()
             Save()
         if version:
-            backupID = packetid.MakeBackupID(customerGlobID, pathID, version)
+            backupID = packetid.MakeBackupID(customerGlobID, pathID, version, key_alias=keyAlias)
             job = GetRunningBackupObject(backupID)
             if job:
                 job.totalSize = sz
@@ -904,18 +773,18 @@ def OnJobDone(backupID, result):
     # from customer import io_throttle
     lg.info('job done [%s] with result "%s", %d more tasks' % (backupID, result, len(tasks())))
     jobs().pop(backupID)
-    customerGlobalID, remotePath, version = packetid.SplitBackupID(backupID)
+    keyAlias, customerGlobalID, remotePath, version = packetid.SplitBackupIDFull(backupID)
     customer_idurl = global_id.GlobalUserToIDURL(customerGlobalID)
     if result == 'done':
         maxBackupsNum = settings.getBackupsMaxCopies()
         if maxBackupsNum:
-            item = backup_fs.GetByID(remotePath, iterID=backup_fs.fsID(customer_idurl))
+            item = backup_fs.GetByID(remotePath, iterID=backup_fs.fsID(customer_idurl, keyAlias))
             if item:
                 versions = item.list_versions(sorted=True, reverse=True)
                 if len(versions) > maxBackupsNum:
                     for version in versions[maxBackupsNum:]:
                         item.delete_version(version)
-                        backupID = packetid.MakeBackupID(customerGlobalID, remotePath, version)
+                        backupID = packetid.MakeBackupID(customerGlobalID, remotePath, version, key_alias=keyAlias)
                         backup_rebuilder.RemoveBackupToWork(backupID)
                         # io_throttle.DeleteBackupRequests(backupID)
                         # io_throttle.DeleteBackupSendings(backupID)
@@ -926,9 +795,9 @@ def OnJobDone(backupID, result):
         backup_fs.ScanID(remotePath)
         backup_fs.Calculate()
         Save()
-        control.request_update([('pathID', remotePath), ])
+        # control.request_update([('pathID', remotePath), ])
         # TODO: check used space, if we have over use - stop all tasks immediately
-        backup_matrix.RepaintBackup(backupID)
+        # backup_matrix.RepaintBackup(backupID)
     elif result == 'abort':
         DeleteBackup(backupID)
     if len(tasks()) == 0:
@@ -945,8 +814,6 @@ def OnJobDone(backupID, result):
 
 
 def OnJobFailed(backupID, err):
-    """
-    """
     lg.err('job failed [%s] : %s' % (backupID, err))
     jobs().pop(backupID)
 
@@ -971,22 +838,16 @@ def OnBackupBlockReport(backupID, blockNum, result):
 
 
 def OnNewDataPrepared():
-    """
-    """
     data_sender.A('new-data')
 
 
 def OnTaskExecutedCallback(result):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'backup_control.OnTaskExecuted %s : %s' % (result[0], result[1]))
     return result
 
 
 def OnTaskFailedCallback(result):
-    """
-    """
     lg.err('pathID: %s, error: %s' % (result[0], result[1]))
     return result
 
@@ -1071,7 +932,8 @@ def StartRecursive(pathID, keyID=None):
     backup_fs.TraverseByID(visitor)
     reactor.callLater(0, RunTask)  # @UndefinedVariable
     reactor.callLater(0, backup_monitor.A, 'restart')  # @UndefinedVariable
-    lg.out(6, 'backup_control.StartRecursive %s  :  %d tasks started' % (pathID, len(startedtasks)))
+    if _Debug:
+        lg.out(_DebugLevel, 'backup_control.StartRecursive %s  :  %d tasks started' % (pathID, len(startedtasks)))
     return startedtasks
 
 #------------------------------------------------------------------------------
@@ -1097,8 +959,6 @@ def IsPathInProcess(pathID):
 
 
 def FindRunningBackup(pathID=None, customer=None):
-    """
-    """
     if pathID:
         pathID = global_id.CanonicalID(pathID)
     result = set()
