@@ -82,6 +82,7 @@ from lib import misc
 from lib import strng
 
 from main import settings
+from main import listeners
 
 from contacts import contactsdb
 
@@ -123,7 +124,7 @@ def init():
     """
     if _Debug:
         lg.out(_DebugLevel, 'backup_matrix.init')
-    RepaintingProcess(True)
+    # RepaintingProcess(True)
     ReadLocalFiles()
     ReadLatestRawListFiles()
 
@@ -134,7 +135,7 @@ def shutdown():
     """
     if _Debug:
         lg.out(_DebugLevel, 'backup_matrix.shutdown')
-    RepaintingProcess(False)
+    # RepaintingProcess(False)
 
 #------------------------------------------------------------------------------
 
@@ -234,18 +235,25 @@ def SuppliersChangedNumbers(oldSupplierList, customer_idurl=None):
     """
     changedList = []
     for i in range(len(oldSupplierList)):
-        suplier_idurl = oldSupplierList[i]
-        if not suplier_idurl:
+        old_supplier_idurl = oldSupplierList[i]
+        new_supplier_idurl = contactsdb.supplier(i, customer_idurl=customer_idurl)
+        if not old_supplier_idurl:
+            if new_supplier_idurl:
+                changedList.append(i)
+                lg.info('found empty supplier on position %d replaced with new supplier %r' % (i, new_supplier_idurl, ))
+            else:
+                lg.warn('found empty supplier on position %d which was not replaced yet' % i)
             continue
-        if contactsdb.supplier(i, customer_idurl=customer_idurl) != suplier_idurl:
+        if not id_url.is_the_same(new_supplier_idurl, old_supplier_idurl):
             changedList.append(i)
+            lg.info('found supplier change on position %d: %r -> %r' % (i, old_supplier_idurl, new_supplier_idurl, ))
+    if _Debug:
+        lg.args(_DebugLevel, changed=changedList, old_suppliers=oldSupplierList)
     return changedList
 
 #------------------------------------------------------------------------------
 
 def DetectSupplierPosition(raw_list_file_text):
-    """
-    """
     inpt = BytesIO(strng.to_bin(raw_list_file_text))
     all_positions = {}
     while True:
@@ -318,7 +326,7 @@ def process_line_file(line, current_key_alias=None, customer_idurl=None, is_in_s
         pth = line
         filesz = -1
     if auto_create:
-        if not backup_fs.IsFileID(pth, iterID=backup_fs.fsID(customer_idurl)):
+        if not backup_fs.IsFileID(pth, iterID=backup_fs.fsID(customer_idurl, current_key_alias)):
             if _Debug:
                 lg.out(_DebugLevel, '        AUTO CREATE FILE "%s" in the index' % pth)
             if pth.strip('/') not in [settings.BackupIndexFileName(), ]:
@@ -329,13 +337,9 @@ def process_line_file(line, current_key_alias=None, customer_idurl=None, is_in_s
                     key_id=global_id.MakeGlobalID(idurl=customer_idurl, key_alias=current_key_alias) if current_key_alias else None,
                 )
                 item.size = filesz
-                backup_fs.SetFile(
-                    item,
-                    iter=backup_fs.fs(customer_idurl),
-                    iterID=backup_fs.fsID(customer_idurl),
-                )
+                backup_fs.SetFile(item, customer_idurl=customer_idurl)
                 modified = True
-    if not backup_fs.IsFileID(pth, iterID=backup_fs.fsID(customer_idurl)):
+    if not backup_fs.IsFileID(pth, iterID=backup_fs.fsID(customer_idurl, current_key_alias)):
         # remote supplier have some file - but we don't have it in the index
         if pth.strip('/') in [settings.BackupIndexFileName(), ]:
             # this is the index file saved on remote supplier
@@ -344,14 +348,10 @@ def process_line_file(line, current_key_alias=None, customer_idurl=None, is_in_s
                 name=pth.strip('/'),
                 path_id=pth.strip('/'),
                 typ=backup_fs.FILE,
-                # key_id=global_id.MakeGlobalID(idurl=customer_idurl, key_alias=current_key_alias),
+                key_id=global_id.MakeGlobalID(idurl=customer_idurl, key_alias=current_key_alias),
             )
             item.size = filesz
-            backup_fs.SetFile(
-                item,
-                iter=backup_fs.fs(customer_idurl),
-                iterID=backup_fs.fsID(customer_idurl),
-            )
+            backup_fs.SetFile(item, customer_idurl=customer_idurl)
             modified = True
         else:
             if is_in_sync:
@@ -384,7 +384,7 @@ def process_line_dir(line, current_key_alias=None, customer_idurl=None, is_in_sy
     except:
         pth = line
     if auto_create:
-        if not backup_fs.ExistsID(pth, iterID=backup_fs.fsID(customer_idurl)):
+        if not backup_fs.ExistsID(pth, iterID=backup_fs.fsID(customer_idurl, current_key_alias)):
             if _Debug:
                 lg.out(_DebugLevel, '        AUTO CREATE DIR "%s" in the index' % pth)
             if pth.strip('/') not in [settings.BackupIndexFileName(), ]: 
@@ -394,13 +394,9 @@ def process_line_dir(line, current_key_alias=None, customer_idurl=None, is_in_sy
                     typ=backup_fs.DIR,
                     key_id=global_id.MakeGlobalID(idurl=customer_idurl, key_alias=current_key_alias) if current_key_alias else None,
                 )
-                backup_fs.SetDir(
-                    item,
-                    iter=backup_fs.fs(customer_idurl),
-                    iterID=backup_fs.fsID(customer_idurl),
-                )
+                backup_fs.SetDir(item, customer_idurl=customer_idurl)
                 modified = True
-    if not backup_fs.ExistsID(pth, iterID=backup_fs.fsID(customer_idurl)):
+    if not backup_fs.ExistsID(pth, iterID=backup_fs.fsID(customer_idurl, current_key_alias)):
         if is_in_sync:
             if customer_idurl == my_id.getIDURL():
                 paths2remove.add(
@@ -455,7 +451,7 @@ def process_line_version(line, supplier_num, current_key_alias=None, customer_id
         if _Debug:
             lg.out(_DebugLevel, '        VERSION "%s" to be removed, different supplier number' % backupID)
         return modified, backups2remove, paths2remove, found_backups, newfiles
-    iter_path = backup_fs.WalkByID(remotePath, iterID=backup_fs.fsID(customer_idurl))
+    iter_path = backup_fs.WalkByID(remotePath, iterID=backup_fs.fsID(customer_idurl, current_key_alias))
     item = None
     if iter_path:
         item = iter_path[0]
@@ -561,7 +557,7 @@ def process_line_version(line, supplier_num, current_key_alias=None, customer_id
         item.set_version_info(versionName, maxBlockNum, versionSize)
         modified = True
     # mark this backup to be repainted
-    RepaintBackup(backupID)
+    # RepaintBackup(backupID)
     return modified, backups2remove, paths2remove, found_backups, newfiles
 
 
@@ -598,12 +594,12 @@ def process_raw_list_files(supplier_num, list_files_text_body, customer_idurl=No
     if is_in_sync is None:
         if driver.is_on('service_backup_db'):
             from storage import index_synchronizer
-            is_in_sync = index_synchronizer.is_synchronized() and backup_control.revision() > 0
+            is_in_sync = index_synchronizer.is_synchronized() and backup_fs.revision() > 0
         else:
             is_in_sync = False
     if _Debug:
         lg.out(_DebugLevel, 'backup_matrix.process_raw_list_files [%d] : %d bytes, is_in_sync=%s, rev:%d, customer_idurl=%s' % (
-            supplier_num, len(list_files_text_body), is_in_sync, backup_control.revision(), customer_idurl))
+            supplier_num, len(list_files_text_body), is_in_sync, backup_fs.revision(), customer_idurl))
     backups2remove = set()
     paths2remove = set()
     missed_backups = set(remote_files().keys())
@@ -613,6 +609,7 @@ def process_raw_list_files(supplier_num, list_files_text_body, customer_idurl=No
     current_key_alias = 'master'
     current_query = None
     query_results = set()
+    updated_keys = []
     inpt = BytesIO(strng.to_bin(list_files_text_body))
     while True:
         line = strng.to_text(inpt.readline())
@@ -638,6 +635,8 @@ def process_raw_list_files(supplier_num, list_files_text_body, customer_idurl=No
             continue
 
         if typ == 'D':
+            if current_key_alias == 'master' and not id_url.is_the_same(customer_idurl, my_id.getIDURL()):
+                continue
             modified, _paths2remove = process_line_dir(
                 line,
                 current_key_alias=current_key_alias,
@@ -647,9 +646,13 @@ def process_raw_list_files(supplier_num, list_files_text_body, customer_idurl=No
             )
             paths2remove.update(_paths2remove)
             remote_files_changed = remote_files_changed or modified
+            if modified:
+                updated_keys.append(current_key_alias)
             continue
 
         if typ == 'F':
+            if current_key_alias == 'master' and not id_url.is_the_same(customer_idurl, my_id.getIDURL()):
+                continue
             modified, _paths2remove = process_line_file(
                 line,
                 current_key_alias=current_key_alias,
@@ -659,9 +662,13 @@ def process_raw_list_files(supplier_num, list_files_text_body, customer_idurl=No
             )
             paths2remove.update(_paths2remove)
             remote_files_changed = remote_files_changed or modified
+            if modified:
+                updated_keys.append(current_key_alias)
             continue
 
         if typ == 'V':
+            if current_key_alias == 'master' and not id_url.is_the_same(customer_idurl, my_id.getIDURL()):
+                continue
             modified, _backups2remove, _paths2remove, found_backups, _newfiles = process_line_version(
                 line,
                 supplier_num=supplier_num,
@@ -677,16 +684,20 @@ def process_raw_list_files(supplier_num, list_files_text_body, customer_idurl=No
             remote_files_changed = remote_files_changed or modified
             if current_query is not None:
                 query_results.add((customer_idurl, current_query))
+            if modified:
+                updated_keys.append(current_key_alias)
             continue
 
         raise Exception('unexpected line received: %r' % line)
 
     inpt.close()
+    remote_files_changed = remote_files_changed or (oldfiles != newfiles)
     if _Debug:
-        lg.out(_DebugLevel, 'backup_matrix.process_raw_list_files remote_files_changed:%s old:%d new:%d backups2remove:%d paths2remove:%d missed_backups:%d remote_files:%d query_results:%d' % (
+        lg.out(_DebugLevel, 'backup_matrix.process_raw_list_files   remote_files_changed:%s old:%d new:%d backups2remove:%d paths2remove:%d missed_backups:%d remote_files:%d query_results:%d' % (
             remote_files_changed, oldfiles, newfiles, len(backups2remove), len(paths2remove), len(missed_backups), len(remote_files()), len(query_results), ))
     if remote_files_changed and is_in_sync:
-        backup_control.Save()
+        for key_alias in updated_keys:
+            backup_control.Save(customer_idurl, key_alias)
     for query_key in query_results:
         if query_key in _ListFilesQueryCallbacks:
             for cb in _ListFilesQueryCallbacks[query_key]:
@@ -756,7 +767,7 @@ def ReadLocalFiles():
             return True
         if realpath.startswith('newblock-'):
             return False
-        if subpath in [settings.BackupIndexFileName(), settings.BackupInfoFileName(), settings.BackupInfoFileNameOld(), settings.BackupInfoEncryptedFileName()]:
+        if subpath == settings.BackupIndexFileName():
             return False
         try:
             version = subpath.split('/')[-2]
@@ -797,31 +808,18 @@ def ReadLocalFiles():
 
     if _Debug:
         lg.out(_DebugLevel, 'backup_matrix.ReadLocalFiles %d files indexed' % _counter[0])
-        try:
-            if sys.version_info >= (2, 6):
-                import lib.getsizeof
-                localSZ = lib.getsizeof.total_size(local_files())
-                remoteSZ = lib.getsizeof.total_size(remote_files())
-                indexByName = lib.getsizeof.total_size(backup_fs.fs())
-                indexByID = lib.getsizeof.total_size(backup_fs.fsID())
-                lg.out(_DebugLevel, '    all local info uses %d bytes in the memory' % localSZ)
-                lg.out(_DebugLevel, '    all remote info uses %d bytes in the memory' % remoteSZ)
-                lg.out(_DebugLevel, '    index by name takes %d bytes in the memory' % indexByName)
-                lg.out(_DebugLevel, '    index by ID takes %d bytes in the memory' % indexByID)
-        except:
-            lg.exc()
     if _LocalFilesNotifyCallback is not None:
         _LocalFilesNotifyCallback()
 
 #------------------------------------------------------------------------------
 
 
-def RemoteFileReport(backupID, blockNum, supplierNum, dataORparity, result):
+def RemoteFileReport(backupID, blockNum, supplierNum, dataORparity, result, itemInfo):
     """
     Writes info for a single piece of data into "remote" matrix.
 
     May be called when you got an Ack packet from remote supplier after
-    you sent him some Data packet .
+    you sent him some Data packet.
     """
     blockNum = int(blockNum)
     supplierNum = int(supplierNum)
@@ -845,11 +843,27 @@ def RemoteFileReport(backupID, blockNum, supplierNum, dataORparity, result):
         remote_files()[backupID][blockNum]['P'][supplierNum] = flag
     else:
         lg.warn('incorrect backup ID: %s' % backupID)
-    # if we know only 5 blocks stored on remote machine
-    # but we have backed up 6th block - remember this
-    remote_max_block_numbers()[backupID] = max(remote_max_block_numbers().get(backupID, -1), blockNum)
-    # mark to repaint this backup in gui
-    RepaintBackup(backupID)
+    # if we know only N blocks stored on remote machine
+    # but we uploaded N+1 block - remember that
+    maxBlockNum = max(remote_max_block_numbers().get(backupID, -1), blockNum)
+    remote_max_block_numbers()[backupID] = maxBlockNum
+    # RepaintBackup(backupID)
+    full_remote_path = global_id.MakeGlobalID(path=itemInfo['name'], key_id=itemInfo['key_id'])
+    full_remote_path_id = global_id.MakeGlobalID(path=itemInfo['path_id'], key_id=itemInfo['key_id'])
+    _, percent, _, weakPercent = GetBackupRemoteStats(backupID)
+    listeners.push_snapshot('remote_version', snap_id=backupID, data=dict(
+        backup_id=backupID,
+        max_block=maxBlockNum,
+        remote_path=full_remote_path,
+        global_id=full_remote_path_id,
+        type=itemInfo['type'],
+        size=itemInfo['size'],
+        key_id=itemInfo['key_id'],
+        delivered=misc.percent2string(percent),
+        reliable=misc.percent2string(weakPercent),
+    ))
+    if _Debug:
+        lg.args(_DebugLevel, i=itemInfo['name'], b=backupID, s=supplierNum, n=blockNum, r=result)
 
 
 def LocalFileReport(packetID=None, backupID=None, blockNum=None, supplierNum=None, dataORparity=None):
@@ -908,7 +922,7 @@ def LocalFileReport(packetID=None, backupID=None, blockNum=None, supplierNum=Non
         local_backup_size()[backupID] += os.path.getsize(localDest)
     except:
         lg.exc()
-    RepaintBackup(backupID)
+    # RepaintBackup(backupID)
 
 
 def LocalBlockReport(backupID, blockNumber, result):
@@ -925,7 +939,7 @@ def LocalBlockReport(backupID, blockNumber, result):
         return
     customer, _ = packetid.SplitPacketID(backupID)
     customer_idurl = global_id.GlobalUserToIDURL(customer)
-    repaint_flag = False
+    # repaint_flag = False
     if _Debug:
         lg.out(_DebugLevel, 'backup_matrix.LocalFileReport  in block %d at %s for %s' % (blockNumber, backupID, customer, ))
     num_suppliers = contactsdb.num_suppliers(customer_idurl=customer_idurl)
@@ -940,25 +954,25 @@ def LocalBlockReport(backupID, blockNumber, result):
             local_file = os.path.join(settings.getLocalBackupsDir(), customer, packetID)
             if backupID not in local_files():
                 local_files()[backupID] = {}
-                repaint_flag = True
+                # repaint_flag = True
                 if _Debug:
                     lg.out(_DebugLevel, '    new local entry for %s created in memory' % backupID)
             if blockNum not in local_files()[backupID]:
                 local_files()[backupID][blockNum] = {
                     'D': [0, ] * num_suppliers,
                     'P': [0, ] * num_suppliers}
-                repaint_flag = True
+                # repaint_flag = True
             if not os.path.isfile(local_file):
                 local_files()[backupID][blockNum][dataORparity[0]][supplierNum] = 0
-                repaint_flag = True
+                # repaint_flag = True
                 continue
             local_files()[backupID][blockNum][dataORparity[0]][supplierNum] = 1
             if backupID not in local_backup_size():
                 local_backup_size()[backupID] = 0
-                repaint_flag = True
+                # repaint_flag = True
             try:
                 local_backup_size()[backupID] += os.path.getsize(local_file)
-                repaint_flag = True
+                # repaint_flag = True
             except:
                 lg.exc()
             if _Debug:
@@ -968,8 +982,8 @@ def LocalBlockReport(backupID, blockNumber, result):
         local_max_block_numbers()[backupID] = -1
     if local_max_block_numbers()[backupID] < blockNum:
         local_max_block_numbers()[backupID] = blockNum
-    if repaint_flag:
-        RepaintBackup(backupID)
+    # if repaint_flag:
+    #     RepaintBackup(backupID)
 
 #------------------------------------------------------------------------------
 
@@ -1119,8 +1133,8 @@ def ScanBlocksToSend(backupID, limit_per_supplier=None):
     for supplierNum in range(len(supplierActiveArray)):
         bySupplier[supplierNum] = set()
     if backupID not in remote_files():
-        if _Debug:
-            lg.out(_DebugLevel, 'backup_matrix.ScanBlocksToSend  backupID %r not found in remote files' % backupID)
+        # if _Debug:
+        #     lg.out(_DebugLevel, 'backup_matrix.ScanBlocksToSend  backupID %r not found in remote files' % backupID)
         for blockNum in range(localMaxBlockNum + 1):
             localData = GetLocalDataArray(backupID, blockNum)
             localParity = GetLocalParityArray(backupID, blockNum)
@@ -1137,8 +1151,8 @@ def ScanBlocksToSend(backupID, limit_per_supplier=None):
                     if len(bySupplier[supplierNum]) > limit_per_supplier:
                         break
     else:
-        if _Debug:
-            lg.out(_DebugLevel, 'backup_matrix.ScanBlocksToSend  backupID %r was found in remote files' % backupID)
+        # if _Debug:
+        #     lg.out(_DebugLevel, 'backup_matrix.ScanBlocksToSend  backupID %r was found in remote files' % backupID)
         for blockNum in range(localMaxBlockNum + 1):
             remoteData = GetRemoteDataArray(backupID, blockNum)
             remoteParity = GetRemoteParityArray(backupID, blockNum)
@@ -1266,6 +1280,8 @@ def ClearSupplierRemoteInfo(supplierNum, customer_idurl=None):
                         remote_files()[backupID][blockNum]['P'][supplierNum] = 0
                 except:
                     pass
+    if _Debug:
+        lg.args(_DebugLevel, files_cleaned=files, supplier_pos=supplierNum, customer=customer_idurl)
     return files
 
 #------------------------------------------------------------------------------
@@ -1487,8 +1503,6 @@ def GetKnownMaxBlockNum(backupID):
 
 
 def GetLocalMatrix(backupID, blockNum):
-    """
-    """
     customer_idurl = packetid.CustomerIDURL(backupID)
     if backupID not in local_files():
         return {'D': [0] * contactsdb.num_suppliers(customer_idurl=customer_idurl),
@@ -1526,8 +1540,6 @@ def GetLocalParityArray(backupID, blockNum):
 
 
 def GetRemoteMatrix(backupID, blockNum):
-    """
-    """
     customer_idurl = packetid.CustomerIDURL(backupID)
     if backupID not in remote_files():
         return {'D': [0] * contactsdb.num_suppliers(customer_idurl=customer_idurl),
@@ -1681,6 +1693,28 @@ def remove_list_files_query_callback(customer_idurl, query_path, callback_method
 
 #------------------------------------------------------------------------------
 
+def populate_remote_versions():
+    for backupID in GetBackupIDs(remote=True, local=False, sorted_ids=False):
+        customer_idurl = packetid.CustomerIDURL(backupID)
+        keyAlias, _, pathID, _ = packetid.SplitBackupIDFull(backupID)
+        itemInfo = backup_fs.GetByID(pathID, iterID=backup_fs.fsID(customer_idurl, keyAlias))
+        if itemInfo:
+            full_remote_path = global_id.MakeGlobalID(path=itemInfo.name(), key_id=itemInfo.key_id)
+            full_remote_path_id = global_id.MakeGlobalID(path=itemInfo.path_id, key_id=itemInfo.key_id)
+            _, percent, _, weakPercent = GetBackupRemoteStats(backupID)
+            listeners.push_snapshot('remote_version', snap_id=backupID, data=dict(
+                backup_id=backupID,
+                max_block=remote_max_block_numbers().get(backupID, -1),
+                remote_path=full_remote_path,
+                global_id=full_remote_path_id,
+                type=backup_fs.TYPES.get(itemInfo.type, 'UNKNOWN').lower(),
+                size=itemInfo.size,
+                key_id=itemInfo.key_id,
+                delivered=misc.percent2string(percent),
+                reliable=misc.percent2string(weakPercent),
+            ))
+
+#------------------------------------------------------------------------------
 
 if __name__ == "__main__":
     init()
