@@ -97,30 +97,34 @@ if __name__ == '__main__':
 from logs import lg
 
 from p2p import commands
+from p2p import p2p_service
 
 from lib import nameurl
 from lib import misc
 from lib import strng
+from lib import packetid
 
 from system import bpio
 from system import tmpfile
 
 from main import settings
-from main import control
+# from main import control
 from main import config
 from main import events
 
 from crypt import signed
 
-from userid import my_id
-from userid import global_id
-
 from contacts import identitycache
 
 from transport import callback
 from transport import packet_in
 from transport import packet_out
 
+from userid import global_id
+from userid import identity
+from userid import id_url
+from userid import my_id
+
 #------------------------------------------------------------------------------
 
 _AvailableTransports = {}
@@ -139,8 +143,6 @@ _TransportLogFilename = None
 #------------------------------------------------------------------------------
 
 def init():
-    """
-    """
     global _LocalListener
     global _PacketLogFileEnabled
     if _Debug:
@@ -176,23 +178,17 @@ def shutdown():
 #------------------------------------------------------------------------------
 
 def transport(proto):
-    """
-    """
     global _TransportsDict
     proto = strng.to_text(proto)
     return _TransportsDict[proto]
 
 
 def transports():
-    """
-    """
     global _TransportsDict
     return _TransportsDict
 
 
 def listener():
-    """
-    """
     global _LocalListener
     return _LocalListener
 
@@ -214,22 +210,16 @@ def is_installed(proto):
 
 
 def can_send(proto):
-    """
-    """
     return transport(proto).state == 'LISTENING'
 
 
 def last_inbox_time():
-    """
-    """
     global _LastInboxPacketTime
     return _LastInboxPacketTime
 
 #------------------------------------------------------------------------------
 
 def start():
-    """
-    """
     if _Debug:
         lg.out(4, 'gateway.start')
     callback.append_outbox_filter_callback(on_outbox_packet)
@@ -250,8 +240,6 @@ def start():
 
 
 def cold_start():
-    """
-    """
     if _Debug:
         lg.out(4, 'gateway.cold_start : sending "start" only to one transport - most preferable')
     callback.append_outbox_filter_callback(on_outbox_packet)
@@ -277,8 +265,6 @@ def cold_start():
 #------------------------------------------------------------------------------
 
 def stop():
-    """
-    """
     if _Debug:
         lg.out(4, 'gateway.stop')
     stop_packets_timeout_loop()
@@ -302,8 +288,6 @@ def stop():
 #------------------------------------------------------------------------------
 
 def verify():
-    """
-    """
     ordered_list = list(transports().keys())
     ordered_list.sort(key=settings.getTransportPriority, reverse=True)
     if _Debug:
@@ -359,8 +343,6 @@ def verify():
 
 
 def attach(transport_instance):
-    """
-    """
     global _TransportsDict
     global _AvailableTransports
     proto = strng.to_text(transport_instance.proto)
@@ -371,8 +353,6 @@ def attach(transport_instance):
 
 
 def detach(transport_instance):
-    """
-    """
     global _TransportsDict
     global _AvailableTransports
     proto = strng.to_text(transport_instance.proto)
@@ -533,8 +513,6 @@ def make_transfer_ID():
 #------------------------------------------------------------------------------
 
 def connect_to(proto, host):
-    """
-    """
     if not is_ready():
         return fail(Exception('gateway is not ready'))
     if not is_installed(proto):
@@ -543,8 +521,6 @@ def connect_to(proto, host):
 
 
 def disconnect_from(proto, host):
-    """
-    """
     if not is_ready():
         return fail(Exception('gateway is not ready'))
     if not is_installed(proto):
@@ -578,8 +554,6 @@ def send_file(remote_idurl, proto, host, filename, description='', pkt_out=None)
 
 
 def send_file_single(remote_idurl, proto, host, filename, description='', pkt_out=None):
-    """
-    """
     if not is_ready():
         lg.warn('gateway is not ready')
         return False
@@ -592,8 +566,6 @@ def send_file_single(remote_idurl, proto, host, filename, description='', pkt_ou
 
 
 def send_keep_alive(proto, host):
-    """
-    """
     if not is_ready():
         lg.warn('gateway is not ready')
         return False
@@ -605,8 +577,6 @@ def send_keep_alive(proto, host):
 
 
 def list_active_transports():
-    """
-    """
     if not is_ready():
         return fail(Exception('gateway is not ready'))
     result = []
@@ -618,8 +588,6 @@ def list_active_transports():
 
 
 def list_active_sessions(proto):
-    """
-    """
     if not is_ready():
         return fail(Exception('gateway is not ready'))
     if not is_installed(proto):
@@ -628,8 +596,6 @@ def list_active_sessions(proto):
 
 
 def list_active_streams(proto):
-    """
-    """
     if not is_ready():
         return fail(Exception('gateway is not ready'))
     if not is_installed(proto):
@@ -638,8 +604,6 @@ def list_active_streams(proto):
 
 
 def find_active_session(proto, host=None, idurl=None):
-    """
-    """
     if not is_ready():
         # return fail(Exception('gateway is not ready'))
         lg.warn('gateway is not ready')
@@ -652,8 +616,6 @@ def find_active_session(proto, host=None, idurl=None):
 
 
 def find_active_stream(proto, stream_id=None, transfer_id=None):
-    """
-    """
     if not is_ready():
         lg.warn('gateway is not ready')
         return None
@@ -715,8 +677,6 @@ def current_bytes_received():
 
 
 def shutdown_all_outbox_packets():
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'gateway.shutdown_all_outbox_packets, %d live objects at the moment' % len(packet_out.queue()))
     for pkt_out in list(packet_out.queue()):
@@ -724,8 +684,6 @@ def shutdown_all_outbox_packets():
 
 
 def shutdown_all_inbox_packets():
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel, 'gateway.shutdown_all_inbox_packets, %d live objects at the moment' % len(list(packet_in.inbox_items().values())))
     for pkt_in in list(packet_in.inbox_items().values()):
@@ -736,7 +694,7 @@ def shutdown_all_inbox_packets():
 
 def packets_timeout_loop():
     global _PacketsTimeOutTask
-    delay = 10
+    delay = 30
     _PacketsTimeOutTask = reactor.callLater(delay, packets_timeout_loop)  # @UndefinedVariable
     for pkt_in in list(packet_in.inbox_items().values()):
         if pkt_in.is_timed_out():
@@ -785,25 +743,92 @@ def on_file_received(info, data):
     return False
 
 
+def on_identity_received(newpacket, send_ack=True):
+    """
+    A normal node or identity server is sending us a new copy of an identity for a contact of ours.
+    Checks that identity is signed correctly.
+    This will be also sending requests to cache all sources (other identity servers) holding that identity.
+    """
+    # TODO:  move to service_gateway
+    newxml = newpacket.Payload
+    newidentity = identity.identity(xmlsrc=newxml)
+    # SECURITY
+    # check that identity is signed correctly
+    # old public key matches new one
+    # this is done in `UpdateAfterChecking()`
+    idurl = newidentity.getIDURL()
+    if not identitycache.HasKey(idurl):
+        lg.info('received new identity %s rev %r' % (idurl, newidentity.getRevisionValue()))
+    if my_id.isLocalIdentityReady():
+        if newidentity.getPublicKey() == my_id.getLocalIdentity().getPublicKey():
+            if newidentity.getRevisionValue() > my_id.getLocalIdentity().getRevisionValue():
+                lg.warn('received my own identity from another user, but with higher revision number')
+                reactor.callLater(0, my_id.rebuildLocalIdentity, new_revision=newidentity.getRevisionValue() + 1)  # @UndefinedVariable
+                return False
+    if not identitycache.UpdateAfterChecking(idurl, newxml):
+        lg.warn("ERROR has non-Valid identity")
+        return False
+    latest_identity = id_url.get_latest_ident(newidentity.getPublicKey())
+    if latest_identity:
+        if latest_identity.getRevisionValue() > newidentity.getRevisionValue():
+            # check if received identity is the most recent revision number we ever saw for that remote user
+            # in case we saw same identity with higher revision number need to reply with Fail packet and notify user
+            # this may happen after identity restore - the user starts counting revision number from 0
+            # but other nodes already store previous copies, user just need to jump to the most recent revision number
+            lg.warn('received new identity with out-dated revision number %d from %r, known revision is %d' % (
+                newidentity.getRevisionValue(), idurl, latest_identity.getRevisionValue(), ))
+            lg.warn('received identity: %r' % newxml)
+            lg.warn('known identity: %r' % latest_identity.serialize())
+            ident_packet = signed.Packet(
+                Command=commands.Identity(),
+                OwnerID=latest_identity.getIDURL(),
+                CreatorID=latest_identity.getIDURL(),
+                PacketID='identity:%s' % packetid.UniqueID(),
+                Payload=latest_identity.serialize(),
+                RemoteID=idurl,
+            )
+            reactor.callLater(0, packet_out.create, outpacket=ident_packet, wide=True, callbacks={}, keep_alive=False)  # @UndefinedVariable
+            return False
+    # Now that we have ID we can check the packet
+    if not newpacket.Valid():
+        # If not valid do nothing
+        lg.warn("not Valid packet from %s" % idurl)
+        return False
+    if not send_ack:
+        if _Debug:
+            lg.dbg(_DebugLevel, "%s  idurl=%s  remoteID=%r  skip sending Ack()" % (
+                newpacket.PacketID, idurl, newpacket.RemoteID))
+        return True
+    if newpacket.OwnerID == idurl:
+        if _Debug:
+            lg.dbg(_DebugLevel, "%s  idurl=%s  remoteID=%r  sending wide Ack()" % (
+                newpacket.PacketID, idurl, newpacket.RemoteID))
+    else:
+        if _Debug:
+            lg.dbg(_DebugLevel, "%s  idurl=%s  remoteID=%r  but packet ownerID=%s   sending wide Ack()" % (
+                newpacket.PacketID, idurl, newpacket.RemoteID, newpacket.OwnerID, ))
+    # wide=True : a small trick to respond to all known contacts of the remote user
+    reactor.callLater(0, p2p_service.SendAck, newpacket, wide=True)  # @UndefinedVariable
+    return True
+
+
 def on_outbox_packet(outpacket, wide, callbacks, target=None, route=None, response_timeout=None, keep_alive=True):
-    """
-    """
     started_packets = packet_out.search_similar_packets(outpacket)
     if started_packets:
         for active_packet, _ in started_packets:
+            if active_packet.Command in [commands.Ack(), commands.Fail(), ]:
+                continue
             if callbacks:
                 for command, cb in callbacks.items():
                     active_packet.set_callback(command, cb)
             lg.warn('skip creating new outbox packet because found similar pending packet: %r' % active_packet)
             return active_packet
     pkt_out = packet_out.create(outpacket, wide, callbacks, target, route, response_timeout, keep_alive)
-    control.request_update([('packet', outpacket.PacketID)])
+    # control.request_update([('packet', outpacket.PacketID)])
     return pkt_out
 
 
 def on_transport_state_changed(transport, oldstate, newstate):
-    """
-    """
     global _TransportStateChangedCallbacksList
     if _Debug:
         lg.out(_DebugLevel - 2, 'gateway.on_transport_state_changed in %r : %s->%s' % (
@@ -816,8 +841,6 @@ def on_transport_state_changed(transport, oldstate, newstate):
 
 
 def on_transport_initialized(proto, xmlrpcurl=None):
-    """
-    """
     transport(proto).automat('transport-initialized', xmlrpcurl)
     events.send('gateway-transport-initialized', data=dict(
         proto=proto,
@@ -827,8 +850,6 @@ def on_transport_initialized(proto, xmlrpcurl=None):
 
 
 def on_receiving_started(proto, host, options_modified=None):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel - 2, 'gateway.on_receiving_started %r host=%r' % (proto.upper(), host))
     transport(proto).automat('receiving-started', (proto, host, options_modified))
@@ -841,8 +862,6 @@ def on_receiving_started(proto, host, options_modified=None):
 
 
 def on_receiving_failed(proto, error_code=None):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel - 2, 'gateway.on_receiving_failed %s    error=[%s]' % (proto.upper(), str(error_code)))
     transport(proto).automat('failed')
@@ -854,8 +873,6 @@ def on_receiving_failed(proto, error_code=None):
 
 
 def on_disconnected(proto, result=None):
-    """
-    """
     if _Debug:
         lg.out(_DebugLevel - 2, 'gateway.on_disconnected %s    result=%s' % (proto.upper(), str(result)))
     if proto in transports():
@@ -868,29 +885,23 @@ def on_disconnected(proto, result=None):
 
 
 def on_start_connecting(host):
-    """
-    """
     return True
 
 
 def on_session_opened(host, remote_user_id):
-    """
-    """
+    pass
 
 
 def on_connection_failed(host, error_message=None):
-    """
-    """
+    pass
 
 
 def on_session_closed(host, remote_user_id, reason=None):
-    """
-    """
+    pass
 
 
 def on_message_received(host, remote_user_id, data):
-    """
-    """
+    pass
 
 
 def on_register_file_sending(proto, host, receiver_idurl, filename, size=0, description=''):
@@ -917,7 +928,7 @@ def on_register_file_sending(proto, host, receiver_idurl, filename, size=0, desc
             pkt_out.description, transfer_id, os.path.basename(filename), proto,
             nameurl.GetName(receiver_idurl), host))
     pkt_out.automat('register-item', (proto, host, filename, transfer_id))
-    control.request_update([('stream', transfer_id)])
+    # control.request_update([('stream', transfer_id)])
     return transfer_id
 
 
@@ -935,7 +946,7 @@ def on_unregister_file_sending(transfer_id, status, bytes_sent, error_message=No
             lg.out(_DebugLevel, '        %s is not found' % str(transfer_id))
         return False
     pkt_out.automat('unregister-item', (transfer_id, status, bytes_sent, error_message))
-    control.request_update([('stream', transfer_id)])
+    # control.request_update([('stream', transfer_id)])
     if status == 'finished':
         if _Debug:
             lg.out(_DebugLevel, '>>> OUT >>> %s (%d) [%s://%s] %s with %d bytes' % (
@@ -948,8 +959,6 @@ def on_unregister_file_sending(transfer_id, status, bytes_sent, error_message=No
 
 
 def on_cancelled_file_sending(proto, host, filename, size, description='', error_message=None):
-    """
-    """
     pkt_out, work_item = packet_out.search(proto, host, filename)
     if pkt_out is None:
         if _Debug:
@@ -957,8 +966,8 @@ def on_cancelled_file_sending(proto, host, filename, size, description='', error
                 proto, host, os.path.basename(filename)))
         return True
     pkt_out.automat('item-cancelled', (proto, host, filename, size, description, error_message, ))
-    if pkt_out.outpacket:
-        control.request_update([('packet', pkt_out.outpacket.PacketID)])
+    # if pkt_out.outpacket:
+    #     control.request_update([('packet', pkt_out.outpacket.PacketID)])
     if _Debug:
         lg.out(_DebugLevel, '>>> OUT >>>  {%s} CANCELLED via [%s] to %s : %s' % (
             os.path.basename(filename), proto, host, error_message, ))
@@ -981,7 +990,7 @@ def on_register_file_receiving(proto, host, sender_idurl, filename, size=0):
             nameurl.GetName(sender_idurl), host))
     incoming_packet = packet_in.create(transfer_id)
     incoming_packet.event('register-item', (proto, host, sender_idurl, filename, size))
-    control.request_update([('stream', transfer_id)])
+    # control.request_update([('stream', transfer_id)])
     return transfer_id
 
 
@@ -1001,7 +1010,7 @@ def on_unregister_file_receiving(transfer_id, status, bytes_received, error_mess
             lg.out(_DebugLevel, '<<< IN <<< (%d) [%s://%s] %s : %s' % (
                 transfer_id, pkt_in.proto, pkt_in.host, status.upper(), error_message))
     pkt_in.automat('unregister-item', (status, bytes_received, error_message))
-    control.request_update([('stream', transfer_id)])
+    # control.request_update([('stream', transfer_id)])
     return True
 
 #------------------------------------------------------------------------------
