@@ -99,15 +99,11 @@ _History = []
 #------------------------------------------------------------------------------
 
 def init():
-    """
-    """
     global _PacketLogFileEnabled
     _PacketLogFileEnabled = config.conf().getBool('logs/packet-enabled')
 
 
 def shutdown():
-    """
-    """
     global _PacketLogFileEnabled
     _PacketLogFileEnabled = False
 
@@ -126,8 +122,6 @@ def increment_packets_counter():
 
 
 def inbox_items():
-    """
-    """
     global _InboxItems
     return _InboxItems
 
@@ -183,7 +177,8 @@ def process(newpacket, info):
     Main entry point where all incoming signed packets are coming from remote peers.
     The main aspect here is to "authenticate" remote node - need to know it identity.
     """
-    from p2p import p2p_service
+    # from p2p import p2p_service
+    from transport import gateway
     from userid import my_id
     if not driver.is_on('service_p2p_hookups'):
         if _Debug:
@@ -209,11 +204,11 @@ def process(newpacket, info):
         d.addErrback(lambda err: lg.err('incoming remote ID is unknown, failed caching remote %s identity: %s' % (newpacket.RemoteID, str(err))) and None)
         return d
     if newpacket.Command == commands.Identity():
-        if newpacket.RemoteID != my_id.getIDURL():
+        if not id_url.is_the_same(newpacket.RemoteID, my_id.getIDURL()):
             if _Debug:
                 lg.out(_DebugLevel, '    incoming Identity is routed to another user')
-            if not p2p_service.Identity(newpacket, send_ack=False):
-                lg.warn('non-valid identity received')
+            if not gateway.on_identity_received(newpacket, send_ack=False):
+                lg.warn('received identity was not processed')
                 return None
             # remote peer sending a valid identity to another peer routed via my machine
             # need to handle that packet - it should be processed by proxy_server
@@ -223,8 +218,8 @@ def process(newpacket, info):
         # because we might not have his identity on hands and so can not verify the packet
         # so we check that his Identity is valid and save it into cache
         # than we check the packet to be valid too.
-        if not p2p_service.Identity(newpacket):
-            lg.warn('non-valid identity received')
+        if not gateway.on_identity_received(newpacket):
+            lg.warn('received identity was not processed')
             return None
     if not identitycache.HasKey(newpacket.CreatorID):
         if _Debug:
