@@ -91,15 +91,15 @@ from system import tmpfile
 from contacts import contactsdb
 from contacts import identitycache
 
-from userid import my_id
-from userid import global_id
-from userid import id_url
-
 from main import settings
 from main import config
 
 from transport import callback
 
+from userid import global_id
+from userid import id_url
+from userid import my_id
+
 #------------------------------------------------------------------------------
 
 _OutboxQueue = []
@@ -108,15 +108,11 @@ _PacketsCounter = 0
 #------------------------------------------------------------------------------
 
 def init():
-    """
-    """
     global _PacketLogFileEnabled
     _PacketLogFileEnabled = config.conf().getBool('logs/packet-enabled')
 
 
 def shutdown():
-    """
-    """
     global _PacketLogFileEnabled
     _PacketLogFileEnabled = False
 
@@ -669,6 +665,7 @@ class PacketOut(automat.Automat):
         self.caching_deferred = identitycache.immediatelyCaching(self.remote_idurl)
         self.caching_deferred.addCallback(self._on_remote_identity_cached)
         self.caching_deferred.addErrback(self._on_remote_identity_cache_failed)
+        self.caching_deferred.addTimeout(60, clock=reactor)
 
     def doSerializeAndWrite(self, *args, **kwargs):
         """
@@ -685,9 +682,12 @@ class PacketOut(automat.Automat):
             os.close(fileno)
             self.filesize = len(self.packetdata)
             if self.filesize < 1024 * 10:
-                self.timeout = 10
+                if self.response_timeout:
+                    self.timeout = self.response_timeout
+                else:
+                    self.timeout = 30
             elif self.filesize > 1024 * 1024:
-                self.timeout = int(self.filesize / float(settings.SendingSpeedLimit()))
+                self.timeout = 5 + int(self.filesize / float(settings.SendingSpeedLimit()))
             else:
                 self.timeout = 300
         except:
