@@ -82,8 +82,6 @@ _Ready = False
 #------------------------------------------------------------------------------
 
 def init():
-    """
-    """
     global _IdentityHistoryDir
     global _KnownUsers
     global _KnownIDURLs
@@ -125,8 +123,8 @@ def init():
                 if not known_id_obj.Valid():
                     raise Exception('identity history in %r is broken, identity is not valid: %r' % (
                         one_user_dir, one_ident_path))
-            except:
-                lg.exc()
+            except Exception as exc:
+                lg.warn(str(exc))
                 for_cleanup.append(one_ident_path)
                 continue
             one_pub_key = known_id_obj.getPublicKey()
@@ -165,8 +163,9 @@ def init():
                         _KnownSources[one_pub_key].append(one_source)
                         if _Debug:
                             lg.out(_DebugLevel, '    new source %r added for %r' % (one_source, one_pub_key[-10:], ))
-    for one_ident_file in for_cleanup:
+    for one_ident_path in for_cleanup:
         if os.path.isfile(one_ident_path):
+            lg.warn('about to erase broken historical identity file: %r' % one_ident_path)
             try:
                 os.remove(one_ident_path)
             except:
@@ -175,8 +174,6 @@ def init():
 
 
 def shutdown():
-    """
-    """
     global _IdentityHistoryDir
     global _KnownIDURLs
     global _KnownUsers
@@ -257,7 +254,7 @@ def identity_cached(new_id_obj):
         user_path = _KnownUsers[pub_key]
         user_identity_files = sorted(map(int, os.listdir(user_path)))
         if len(user_identity_files) == 0:
-            raise Exception('identity history for user %r is broken, public key is known, but no identity files found' % user_name)
+            lg.warn('identity history for user %r is broken, public key is known, but no identity files found' % user_name)
         latest_identity_file_path = ''
         latest_pub_key = None
         latest_revision = -1
@@ -285,30 +282,31 @@ def identity_cached(new_id_obj):
                 latest_revision = one_id_obj.getRevisionValue()
                 latest_identity_file_path = identity_file_path
         xmlsrc = local_fs.ReadBinaryFile(latest_identity_file_path)
-        latest_id_obj = identity.identity(xmlsrc=xmlsrc)
-        if latest_id_obj.getPublicKey() != new_id_obj.getPublicKey():
-            raise Exception('identity history for user %r is broken, public key not matching' % user_name)
-        if latest_id_obj.getIDName() != new_id_obj.getIDName():
-            lg.warn('found another user name in identity history for user %r : %r' % (user_name, latest_id_obj.getIDName()))
-        if new_id_obj.getRevisionValue() in known_revisions:
-            if _Debug:
-                lg.out(_DebugLevel, 'id_url.identity_cached revision %d already known for user %r' % (new_id_obj.getRevisionValue(), user_name))
-        else:
-            latest_sources = latest_id_obj.getSources(as_originals=True)
-            new_sources = new_id_obj.getSources(as_originals=True)
-            if latest_sources == new_sources:
-                local_fs.WriteBinaryFile(latest_identity_file_path, new_id_obj.serialize())
+        if xmlsrc:
+            latest_id_obj = identity.identity(xmlsrc=xmlsrc)
+            if latest_id_obj.getPublicKey() != new_id_obj.getPublicKey():
+                raise Exception('identity history for user %r is broken, public key not matching' % user_name)
+            if latest_id_obj.getIDName() != new_id_obj.getIDName():
+                lg.warn('found another user name in identity history for user %r : %r' % (user_name, latest_id_obj.getIDName()))
+            if new_id_obj.getRevisionValue() in known_revisions:
                 if _Debug:
-                    lg.out(_DebugLevel, 'id_url.identity_cached latest identity sources for user %r did not changed, updated file %r' % (
-                        user_name, latest_identity_file_path))
+                    lg.out(_DebugLevel, 'id_url.identity_cached revision %d already known for user %r' % (new_id_obj.getRevisionValue(), user_name))
             else:
-                next_identity_file = user_identity_files[-1] + 1
-                next_identity_file_path = os.path.join(user_path, strng.to_text(next_identity_file))
-                local_fs.WriteBinaryFile(next_identity_file_path, new_id_obj.serialize())
-                is_identity_rotated = True
-                if _Debug:
-                    lg.out(_DebugLevel, 'id_url.identity_cached identity sources for user %r changed, wrote new item in the history: %r' % (
-                        user_name, next_identity_file_path))
+                latest_sources = latest_id_obj.getSources(as_originals=True)
+                new_sources = new_id_obj.getSources(as_originals=True)
+                if latest_sources == new_sources:
+                    local_fs.WriteBinaryFile(latest_identity_file_path, new_id_obj.serialize())
+                    if _Debug:
+                        lg.out(_DebugLevel, 'id_url.identity_cached latest identity sources for user %r did not changed, updated file %r' % (
+                            user_name, latest_identity_file_path))
+                else:
+                    next_identity_file = user_identity_files[-1] + 1
+                    next_identity_file_path = os.path.join(user_path, strng.to_text(next_identity_file))
+                    local_fs.WriteBinaryFile(next_identity_file_path, new_id_obj.serialize())
+                    is_identity_rotated = True
+                    if _Debug:
+                        lg.out(_DebugLevel, 'id_url.identity_cached identity sources for user %r changed, wrote new item in the history: %r' % (
+                            user_name, next_identity_file_path))
     new_revision = new_id_obj.getRevisionValue()
     new_sources = new_id_obj.getSources(as_originals=True)
     for new_idurl in reversed(new_sources):
@@ -384,7 +382,7 @@ def identity_cached(new_id_obj):
                             new_revision=new_revision,
                         ))
         else:
-            lg.warn('cached out-dated revision %d for %r' % (new_revision, new_sources[0]))
+            lg.warn('cached out-dated revision %d for %r, most recent revision is %d' % (new_revision, new_sources[0], latest_revision, ))
     else:
         if _Debug:
             lg.out(_DebugLevel, 'id_url.identity_cached revision %d for %r' % (new_revision, new_sources[0]))
@@ -636,6 +634,42 @@ def get_latest_revision(idurl):
     return latest_idurl, latest_rev
 
 
+def get_latest_ident(pub_key):
+    global _KnownUsers
+    from userid import identity
+    user_path = _KnownUsers.get(pub_key)
+    if not user_path:
+        return None
+    user_identity_files = sorted(map(int, os.listdir(user_path)))
+    if len(user_identity_files) == 0:
+        lg.warn('identity history is broken, public key is known, but no identity files found')
+    latest_revision = -1
+    latest_ident = None
+    known_revisions = set()
+    for_cleanup = []
+    for id_file in user_identity_files:
+        identity_file_path = os.path.join(user_path, strng.to_text(id_file))
+        xmlsrc = local_fs.ReadBinaryFile(identity_file_path)
+        one_id_obj = identity.identity(xmlsrc=xmlsrc)
+        if not one_id_obj.isCorrect():
+            lg.warn('identity history is broken, identity in the file %r is not correct' % identity_file_path)
+            for_cleanup.append(identity_file_path)
+            continue
+        if not one_id_obj.Valid():
+            lg.warn('identity history is broken, identity in the file %r is not valid' % identity_file_path)
+            for_cleanup.append(identity_file_path)
+            continue
+        if pub_key != one_id_obj.getPublicKey():
+            lg.err('identity history is broken, public key not matching in the file %r' % identity_file_path)
+            for_cleanup.append(identity_file_path)
+            continue
+        known_revisions.add(one_id_obj.getRevisionValue())
+        if one_id_obj.getRevisionValue() > latest_revision:
+            latest_revision = one_id_obj.getRevisionValue()
+            latest_ident = one_id_obj
+    return latest_ident
+
+
 def list_known_idurls(idurl, num_revisions=5, include_revisions=False):
     """
     Return latest known revisions of given `idurl` (as binary strings) and revision numbers as a list of tuples.
